# Generated by Django 3.2.4 on 2021-06-14 16:04

from django.db import migrations
import ia_storage.fields
import ia_storage.storage


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0020_auto_20210614_1556"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="rawdatafile",
            name="InternetArchiveFileField",
        ),
        migrations.AddField(
            model_name="rawdatafile",
            name="download_file_archive",
            field=ia_storage.fields.InternetArchiveFileField(
                blank=True,
                help_text="An archive of the original raw data file downloaded from CAL-ACCESS.",
                max_length=255,
                storage=ia_storage.storage.InternetArchiveStorage,
                upload_to="",
                verbose_name="archive of download file",
            ),
        ),
    ]



# Generated by Django 3.2.13 on 2022-09-09 23:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0025_auto_20220725_1711"),
    ]

    operations = [
        migrations.DeleteModel(
            name="RawDataFile",
        ),
        migrations.DeleteModel(
            name="RawDataVersion",
        ),
    ]



"""
Custom managers for working with CAL-ACCESS data.
"""
import os
import calaccess_raw
from postgres_copy import CopyManager


class CalAccessManager(CopyManager):
    """
    Utilities for accessing the raw data associated with a model.
    """

    def get_csv_name(self):
        """
        Returns the name of the model's CSV data file.
        """
        return "{}.csv".format(self.model._meta.db_table.lower())

    def get_csv_path(self):
        """
        Returns the path to the model's CSV data file.
        """
        return os.path.join(
            calaccess_raw.get_data_directory(), "csv", self.get_csv_name()
        )

    def get_tsv_name(self):
        """
        Returns the name of the model's raw TSV data file.
        """
        return "{}.TSV".format(self.model._meta.db_table)

    def get_tsv_path(self):
        """
        Returns the path to the model's raw TSV data file.
        """
        return os.path.join(
            calaccess_raw.get_data_directory(), "tsv", self.get_tsv_name()
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the admins from submodules and thread them together.
"""
from calaccess_raw.admin.base import BaseAdmin
from calaccess_raw.admin.campaign import (
    CvrSoCdAdmin,
    Cvr2SoCdAdmin,
    CvrCampaignDisclosureCdAdmin,
    Cvr2CampaignDisclosureCdAdmin,
    RcptCdAdmin,
    Cvr3VerificationInfoCdAdmin,
    LoanCdAdmin,
    S401CdAdmin,
    ExpnCdAdmin,
    F495P2CdAdmin,
    DebtCdAdmin,
    S496CdAdmin,
    S497CdAdmin,
    F501502CdAdmin,
    S498CdAdmin,
)
from calaccess_raw.admin.inactive import (
    BallotMeasuresCdAdmin,
    CvrF470CdAdmin,
    FilerTypePeriodsCd,
    LobbyistContributions1CdAdmin,
    LobbyistContributions2CdAdmin,
    LobbyistContributions3CdAdmin,
    LobbyistEmpLobbyist1CdAdmin,
    LobbyistEmpLobbyist2CdAdmin,
    LobbyistEmployer1CdAdmin,
    LobbyistEmployer2CdAdmin,
    LobbyistEmployer3CdAdmin,
    LobbyistEmployerFirms1CdAdmin,
    LobbyistEmployerFirms2CdAdmin,
    LobbyistEmployerHistoryCdAdmin,
    LobbyistFirm1CdAdmin,
    LobbyistFirm2CdAdmin,
    LobbyistFirm3CdAdmin,
    LobbyistFirmEmployer1CdAdmin,
    LobbyistFirmEmployer2CdAdmin,
    LobbyistFirmHistoryCdAdmin,
    LobbyistFirmLobbyist1CdAdmin,
    LobbyistFirmLobbyist2CdAdmin,
    EfsFilingLogCdAdmin,
)
from calaccess_raw.admin.lobbying import (
    CvrRegistrationCdAdmin,
    Cvr2RegistrationCdAdmin,
    CvrLobbyDisclosureCdAdmin,
    Cvr2LobbyDisclosureCdAdmin,
    LobbyAmendmentsCdAdmin,
    F690P2CdAdmin,
    LattCdAdmin,
    LexpCdAdmin,
    LccmCdAdmin,
    LothCdAdmin,
    LempCdAdmin,
    LpayCdAdmin,
    LobbyingChgLogCdAdmin,
)
from calaccess_raw.admin.common import (
    FilernameCdAdmin,
    FilerFilingsCdAdmin,
    FilingsCdAdmin,
    SmryCdAdmin,
    CvrE530CdAdmin,
    SpltCdAdmin,
    TextMemoCdAdmin,
    AcronymsCdAdmin,
    AddressCdAdmin,
    FilersCdAdmin,
    FilerAcronymsCdAdmin,
    FilerAddressCdAdmin,
    FilerEthicsClassCdAdmin,
    FilerInterestsCdAdmin,
    FilerLinksCdAdmin,
    FilerStatusTypesCdAdmin,
    FilerToFilerTypeCdAdmin,
    FilerTypesCdAdmin,
    FilerXrefCdAdmin,
    FilingPeriodCdAdmin,
    GroupTypesCdAdmin,
    HeaderCdAdmin,
    HdrCdAdmin,
    ImageLinksCdAdmin,
    LegislativeSessionsCdAdmin,
    LookupCodesCdAdmin,
    NamesCdAdmin,
    ReceivedFilingsCdAdmin,
    ReportsCdAdmin,
)

__all__ = (
    "BaseAdmin",
    "CvrSoCdAdmin",
    "Cvr2SoCdAdmin",
    "CvrCampaignDisclosureCdAdmin",
    "Cvr2CampaignDisclosureCdAdmin",
    "RcptCdAdmin",
    "Cvr3VerificationInfoCdAdmin",
    "LoanCdAdmin",
    "S401CdAdmin",
    "ExpnCdAdmin",
    "F495P2CdAdmin",
    "DebtCdAdmin",
    "S496CdAdmin",
    "SpltCdAdmin",
    "S497CdAdmin",
    "F501502CdAdmin",
    "S498CdAdmin",
    "CvrF470CdAdmin",
    "CvrRegistrationCdAdmin",
    "Cvr2RegistrationCdAdmin",
    "CvrLobbyDisclosureCdAdmin",
    "Cvr2LobbyDisclosureCdAdmin",
    "LobbyAmendmentsCdAdmin",
    "F690P2CdAdmin",
    "LattCdAdmin",
    "LexpCdAdmin",
    "LccmCdAdmin",
    "LothCdAdmin",
    "LempCdAdmin",
    "LpayCdAdmin",
    "FilerFilingsCdAdmin",
    "FilingsCdAdmin",
    "SmryCdAdmin",
    "CvrE530CdAdmin",
    "TextMemoCdAdmin",
    "AcronymsCdAdmin",
    "AddressCdAdmin",
    "BallotMeasuresCdAdmin",
    "EfsFilingLogCdAdmin",
    "FilernameCdAdmin",
    "FilersCdAdmin",
    "FilerAcronymsCdAdmin",
    "FilerAddressCdAdmin",
    "FilerEthicsClassCdAdmin",
    "FilerInterestsCdAdmin",
    "FilerLinksCdAdmin",
    "FilerStatusTypesCdAdmin",
    "FilerToFilerTypeCdAdmin",
    "FilerTypesCdAdmin",
    "FilerXrefCdAdmin",
    "FilingPeriodCdAdmin",
    "FilerTypePeriodsCd",
    "GroupTypesCdAdmin",
    "HeaderCdAdmin",
    "HdrCdAdmin",
    "ImageLinksCdAdmin",
    "LegislativeSessionsCdAdmin",
    "LobbyingChgLogCdAdmin",
    "LobbyistContributions1CdAdmin",
    "LobbyistContributions2CdAdmin",
    "LobbyistContributions3CdAdmin",
    "LobbyistEmployer1CdAdmin",
    "LobbyistEmployer2CdAdmin",
    "LobbyistEmployer3CdAdmin",
    "LobbyistEmployerFirms1CdAdmin",
    "LobbyistEmployerFirms2CdAdmin",
    "LobbyistEmpLobbyist1CdAdmin",
    "LobbyistEmpLobbyist2CdAdmin",
    "LobbyistFirm1CdAdmin",
    "LobbyistFirm2CdAdmin",
    "LobbyistFirm3CdAdmin",
    "LobbyistFirmEmployer1CdAdmin",
    "LobbyistFirmEmployer2CdAdmin",
    "LobbyistFirmLobbyist1CdAdmin",
    "LobbyistFirmLobbyist2CdAdmin",
    "LobbyistFirmHistoryCdAdmin",
    "LobbyistEmployerHistoryCdAdmin",
    "LookupCodesCdAdmin",
    "NamesCdAdmin",
    "ReceivedFilingsCdAdmin",
    "ReportsCdAdmin",
)



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-26 14:49
from __future__ import unicode_literals

import calaccess_raw.annotations
import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0004_auto_20160804_1758"),
    ]

    operations = [
        migrations.AlterField(
            model_name="receivedfilingscd",
            name="filing_directory",
            field=calaccess_raw.fields.CharField(
                db_column="FILING_DIRECTORY",
                help_text="This field is undocumented",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="receivedfilingscd",
            name="filing_file_name",
            field=calaccess_raw.fields.CharField(
                db_column="FILING_FILE_NAME",
                help_text="The field is undocumented",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="receivedfilingscd",
            name="form_id",
            field=calaccess_raw.fields.CharField(
                blank=True,
                choices=[
                    (
                        b"F400",
                        b"Form 400: Statement of Organization (Slate Mailer Organization)",
                    ),
                    (
                        b"F401",
                        b"Form 401: Slate Mailer Organization Campaign Statement",
                    ),
                    (
                        b"F402",
                        b"Form 402: Statement of Termination (Slate Mailer Organization)",
                    ),
                    (
                        b"F410",
                        b"Form 410: Statement of Organization Recipient Committee",
                    ),
                    (b"F425", b"Form 425: Semi-Annual Statement of no Activity"),
                    (
                        b"F450",
                        b"Form 450: Recipient Committee Campaign Disclosure Statement - Short Form",
                    ),
                    (b"F460", b"Form 460: Recipient Committee Campaign Statement"),
                    (
                        b"F461",
                        b"Form 461: Independent Expenditure Committee & Major Donor Committee Campaign Statement",
                    ),
                    (b"F465", b"Form 465: Supplemental Independent Expenditure Report"),
                    (b"F496", b"Form 496: Late Independent Expenditure Report"),
                    (b"F497", b"Form 497: Late Contribution Report"),
                    (b"F498", b"Form 498: Slate Mailer Late Payment Report"),
                    (b"F601", b"Form 601: Lobbying Firm Registration Statement"),
                    (b"F602", b"Form 602: Lobbying Firm Activity Authorization"),
                    (
                        b"F603",
                        b"Form 603: Lobbyist Employer or Lobbying Coalition Registration Statement",
                    ),
                    (b"F604", b"Form 604: Lobbyist Certification Statement"),
                    (b"F606", b"Form 606: Notice of Termination"),
                    (b"F607", b"Form 607: Notice of Withdrawal"),
                    (b"F615", b"Form 615: Lobbyist Report"),
                    (b"F625", b"Form 625: Report of Lobbying Firm"),
                    (
                        b"F635",
                        b"Form 635: Report of Lobbyist Employer or Report of Lobbying Coalition",
                    ),
                    (b"F645", b"Form 645: Report of Person Spending $5,000 or More"),
                ],
                db_column="FORM_ID",
                documentcloud_pages=[
                    calaccess_raw.annotations.DocumentCloud(
                        end_page=8, id="2711624-Overview", start_page=4
                    )
                ],
                help_text="Form identification code",
                max_length=7,
                verbose_name="form identification code",
            ),
        ),
        migrations.AlterField(
            model_name="receivedfilingscd",
            name="receive_comment",
            field=calaccess_raw.fields.CharField(
                db_column="RECEIVE_COMMENT", help_text="A comment", max_length=120
            ),
        ),
    ]



"""
Tests for custom database fields.
"""
# Testing
from django.test import TestCase

# Stuff to test
from calaccess_raw import fields
from calaccess_raw import models

# Logging
import logging

logger = logging.getLogger(__name__)


class FieldTestCase(TestCase):
    """
    Tests related to our custom database fields.
    """

    multi_db = True

    def test_fields(self):
        """
        Creates the fields and tests their attributes.
        """
        f = fields.CharField(help_text="foo")
        self.assertEqual(f.definition(), "Foo")
        f2 = fields.CharField()
        self.assertEqual(f2.definition(), "")

        m = models.CvrCampaignDisclosureCd()
        f3 = m.get_field_list()[1]
        f3.is_unique_key()
        f4 = m.get_field_list()[2]
        f4.is_unique_key()
        f4.description()

        m2 = models.FilerToFilerTypeCd()
        f5 = m2.get_field_list()[2]
        f5.copy_type
        f5.copy_template
        f5.description()



"""
Custom field overrides.

Allow for cleaning and transforming the data when it is loaded into the database with PostgreSQL's COPY command.
"""
from django.db.models import fields
from django.db.models import ForeignKey
from django.template.defaultfilters import capfirst


class CalAccessFieldMixin(fields.Field):
    """
    A set of common helpers for all of our custom fields.
    """

    def definition(self):
        """
        A humanized definition of what's the in field for documentation.
        """
        if self.help_text:
            return capfirst(self.help_text)
        else:
            return ""

    def is_unique_key(self):
        """
        Tests if the column is part of its model's unique key.

        Returns True or False
        """
        if self.__dict__["db_column"] in self.model().get_unique_key_list():
            return True
        return False


class DocumentCloudMixin(fields.Field):
    """
    Adds a documentcloud_pages keyword argument to the Field.

    Allows it to link to documents that explain the contents of the data.
    """

    def __init__(self, documentcloud_pages=[], *args, **kwargs):
        """
        Overrides the standard __init__ to add our documentcloud_page_urls option.
        """
        self.documentcloud_pages = documentcloud_pages
        super(DocumentCloudMixin, self).__init__(*args, **kwargs)

    def deconstruct(self):
        """
        Overrides the standard deconstruct method to add our documentcloud_page_urls option.
        """
        name, path, args, kwargs = super(DocumentCloudMixin, self).deconstruct()
        # Only include kwarg if it's not the default
        if self.documentcloud_pages != []:
            kwargs["documentcloud_pages"] = self.documentcloud_pages
        return name, path, args, kwargs


class CharField(fields.CharField, CalAccessFieldMixin, DocumentCloudMixin):
    """
    A custom character field.
    """

    copy_type = "text"
    copy_template = """
    CASE
        WHEN "%(name)s" IS NULL
            THEN ''
        ELSE regexp_replace(TRIM("%(name)s"), '\r|\n', '', 'g')
    END"""

    def description(self):
        """
        Returns a custom description for documentation that includes the max length.
        """
        return super(CharField, self).description % dict(max_length=self.max_length)


class DateField(fields.DateField, CalAccessFieldMixin):
    """
    A custom date field.
    """

    copy_type = "text"
    copy_template = """
    CASE
        WHEN "%(name)s" IS NOT NULL AND TRIM("%(name)s") != ''
            THEN to_date(substring("%(name)s" from 1 for 10), 'MM/DD/YYYY')
        ELSE null
    END"""


class DateTimeField(fields.DateTimeField, CalAccessFieldMixin):
    """
    A custom datetime field.
    """

    copy_type = "text"
    copy_template = """
    CASE
        WHEN "%(name)s" IS NOT NULL AND TRIM("%(name)s") != ''
            THEN to_timestamp("%(name)s", 'MM/DD/YYYY HH12:MI:SS AM')
        ELSE null
    END"""


class DecimalField(fields.DecimalField, CalAccessFieldMixin):
    """
    A custom decimal field.
    """

    copy_type = "text"
    copy_template = """
    CASE
        WHEN "%(name)s" = ''
            THEN 0.0
        WHEN "%(name)s" IS NULL
            THEN 0.0
        WHEN "%(name)s" IS NOT NULL
            THEN "%(name)s"::numeric
    END"""


class FloatField(fields.FloatField, CalAccessFieldMixin):
    """
    A custom float field.
    """

    copy_type = "text"
    copy_template = """
    CASE
        WHEN TRIM("%(name)s") = ''
            THEN 0.0
        WHEN TRIM("%(name)s") IS NULL
            THEN 0.0
        WHEN TRIM("%(name)s") IS NOT NULL
            THEN "%(name)s"::double precision
    END"""


class IntegerField(fields.IntegerField, CalAccessFieldMixin, DocumentCloudMixin):
    """
    A custom integer field.
    """

    copy_type = "text"
    copy_template = """
    CASE
        WHEN TRIM("%(name)s") = ''
            THEN NULL
        WHEN "%(name)s" = '          '
            THEN NULL
        WHEN "%(name)s" = '         '
            THEN NULL
        WHEN "%(name)s" = 'Y'
            THEN 1
        WHEN "%(name)s" = 'y'
            THEN 1
        WHEN "%(name)s" = 'X'
            THEN 1
        WHEN "%(name)s" = 'x'
            THEN 1
        WHEN "%(name)s" = 'N'
            THEN 0
        WHEN "%(name)s" = 'n'
            THEN 0
        WHEN "%(name)s" IS NOT NULL
            THEN "%(name)s"::int
        ELSE NULL
    END"""


class ForeignKeyField(ForeignKey, CalAccessFieldMixin, DocumentCloudMixin):
    """
    A custom foreign key field.
    """

    @property
    def copy_type(self):
        """
        Returns the copy_type of the related foreign key field.
        """
        return self.foreign_related_fields[0].copy_type

    @property
    def copy_template(self):
        """
        Returns the copy_template of the related foreign key field.
        """
        return self.foreign_related_fields[0].copy_template

    def description(self):
        """
        Returns a description for documentation from the related foreign key field.
        """
        return self.foreign_related_fields[0].description



"""
Download the latest CAL-ACCESS database ZIP.
"""
import os
import shutil

import requests
from calaccess_raw.management.commands import CalAccessCommand


class Command(CalAccessCommand):
    """
    Download the latest CAL-ACCESS database ZIP.
    """

    help = "Download the latest CAL-ACCESS database ZIP"

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # flush previous download
        if os.path.exists(self.download_dir):
            shutil.rmtree(self.download_dir)
        os.mkdir(self.download_dir)

        # Stream the download
        self.header("Downloading ZIP file")
        headers = {
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 12_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36"
        }
        with requests.get(self.url, stream=True, headers=headers, verify=False) as r:
            r.raise_for_status()
            chunk_size = 1024
            with open(self.zip_path, "ab") as fp:
                for chunk in r.iter_content(chunk_size=chunk_size):
                    fp.write(chunk)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing lobbying activity tables from the CAL-ACCESS database.
"""
# Models
from django.db import models
from calaccess_raw import fields
from .base import CalAccessBaseModel

# Annotations
from calaccess_raw import annotations
from calaccess_raw.annotations import DocumentCloud


class CvrRegistrationCd(CalAccessBaseModel):
    """
    The cover page of lobbying-registration forms.
    """

    UNIQUE_KEY = ("FILING_ID", "AMEND_ID", "REC_TYPE", "FORM_TYPE")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=8),
        DocumentCloud(id="2711614", start_page=35, end_page=39),
        DocumentCloud(id="2711616", start_page=22, end_page=27),
        DocumentCloud(id="2712033", start_page=68, end_page=71),
        DocumentCloud(id="2712034", start_page=82, end_page=86),
    ]
    FILING_FORMS = [
        annotations.get_form("F601"),
        annotations.get_form("F602"),
        annotations.get_form("F603"),
        annotations.get_form("F604"),
        annotations.get_form("F606"),
        annotations.get_form("F607"),
    ]
    # a_b_adr1 = fields.CharField(
    #   max_length=55,
    #   db_column='A_B_ADR1',
    #   blank=True,
    #   help_text="First line of individual or business entity address"
    # )
    # a_b_adr2 = fields.CharField(
    #   max_length=55,
    #   db_column='A_B_ADR2',
    #   blank=True,
    #   help_text="Second line of individual or business entity address"
    # )
    a_b_city = fields.CharField(
        max_length=30,
        db_column="A_B_CITY",
        blank=True,
        help_text="Individual or business entity city",
    )
    a_b_name = fields.CharField(
        max_length=200,
        db_column="A_B_NAME",
        blank=True,
        help_text="Name of individual or business entity",
    )
    a_b_st = fields.CharField(
        max_length=2,
        db_column="A_B_ST",
        blank=True,
        help_text="Individual or business entity state",
    )
    a_b_zip4 = fields.CharField(
        max_length=10,
        db_column="A_B_ZIP4",
        blank=True,
        help_text="Individual or business entity ZIP Code.",
    )
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment identification number. A number of 0 is the \
original filing and 1 to 999 amendments.",
        verbose_name="amendment ID",
    )
    # auth_adr1 = fields.CharField(
    #   max_length=55,
    #   db_column='AUTH_ADR1',
    #   blank=True,
    #   help_text='First line of authorized lobbying firm business address',
    # )
    # auth_adr2 = fields.CharField(
    #   max_length=55,
    #   db_column='AUTH_ADR2',
    #   blank=True,
    #   help_text='Second line of authorized lobbying firm business address',
    # )
    auth_city = fields.CharField(
        max_length=30,
        db_column="AUTH_CITY",
        blank=True,
        help_text="Authorized lobbying firm business address city",
    )
    auth_name = fields.CharField(
        max_length=200,
        db_column="AUTH_NAME",
        blank=True,
        help_text="Authorized lobbying firm business name. \
Applies to Form 602.",
    )
    auth_st = fields.CharField(
        max_length=2,
        db_column="AUTH_ST",
        blank=True,
        help_text="Authorized lobbying firm business address state",
    )
    auth_zip4 = fields.CharField(
        max_length=10,
        db_column="AUTH_ZIP4",
        blank=True,
        help_text="Authorized lobbying firm business address ZIP Code",
    )
    # bus_adr1 = fields.CharField(
    #   max_length=55,
    #   db_column='BUS_ADR1',
    #   blank=True,
    #   help_text='First line of filer business street address',
    # )
    # bus_adr2 = fields.CharField(
    #   max_length=55,
    #   db_column='BUS_ADR2',
    #   blank=True,
    #   help_text='Second line of filer business street address',
    # )
    bus_cb = fields.CharField(
        max_length=1,
        db_column="BUS_CB",
        blank=True,
        help_text="Business included activity checkbox",
    )
    bus_city = fields.CharField(
        max_length=30,
        db_column="BUS_CITY",
        help_text="Filer business address city",
    )
    BUS_CLASS_CHOICES = (
        ("ENT", "Entertainment/Recreation"),
        ("FIN", "Finance/Insurance"),
        ("LOG", "Lodging/Restaurants"),
        ("MAN", "Manufacturing/Industrial"),
        ("MER", "Merchandise/Retail"),
        ("OIL", "Oil and Gas"),
        ("OTH", "Other"),
        ("PRO", "Professional/Trade"),
        ("REA", "Real Estate"),
        ("TRN", "Transportation"),
    )
    bus_class = fields.CharField(
        max_length=3,
        db_column="BUS_CLASS",
        blank=True,
        choices=BUS_CLASS_CHOICES,
        help_text="Classifiction values of business related entities. \
This field is exclusive of the business class field. One these \
must be populated but not both.",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=70),
            DocumentCloud(id="2712034", start_page=82),
        ],
    )
    bus_descr = fields.CharField(
        max_length=100,
        db_column="BUS_DESCR",
        blank=True,
        help_text="Description of business classification if coded as other",
    )
    bus_email = fields.CharField(
        max_length=60,
        db_column="BUS_EMAIL",
        blank=True,
        help_text="Filer business address email",
    )
    bus_fax = fields.CharField(
        max_length=20,
        db_column="BUS_FAX",
        blank=True,
        help_text="Filer business address fax number",
    )
    bus_phon = fields.CharField(
        max_length=20,
        db_column="BUS_PHON",
        blank=True,
        help_text="Filer business address phone number",
    )
    bus_st = fields.CharField(
        max_length=2,
        db_column="BUS_ST",
        blank=True,
        help_text="Filer business address state",
    )
    bus_zip4 = fields.CharField(
        max_length=10,
        db_column="BUS_ZIP4",
        blank=True,
        help_text="Filer business address ZIP Code",
    )
    c_less50 = fields.CharField(
        max_length=1,
        db_column="C_LESS50",
        blank=True,
        help_text="Industry associations with fewer than 50 members \
check this box",
    )
    c_more50 = fields.CharField(
        max_length=1,
        db_column="C_MORE50",
        blank=True,
        help_text="Industry associations with more than 50 \
check this box.",
    )
    complet_dt = fields.DateField(
        null=True,
        db_column="COMPLET_DT",
        blank=True,
        help_text="Ethics orientation class completion date. Applies to \
Form 604. As filed by the lobbyist.",
    )
    descrip_1 = fields.CharField(
        max_length=300,
        db_column="DESCRIP_1",
        blank=True,
        help_text="Description of business activity, industry or other",
    )
    descrip_2 = fields.CharField(
        max_length=300,
        db_column="DESCRIP_2",
        blank=True,
        help_text="Description of specific or other lobbying interest",
    )
    eff_date = fields.DateField(
        null=True,
        db_column="EFF_DATE",
        blank=True,
        help_text="Effective date of authoarization or termination",
    )
    ENTITY_CODE_CHOICES = (
        ("BUS", "Unknown"),
        ("FRM", annotations.choices.LOBBYING_ENTITY_CODES["FRM"]),
        ("LBY", annotations.choices.LOBBYING_ENTITY_CODES["LBY"]),
        ("LCO", annotations.choices.LOBBYING_ENTITY_CODES["LCO"]),
        ("LEM", annotations.choices.LOBBYING_ENTITY_CODES["LEM"]),
    )
    entity_cd = fields.CharField(
        max_length=3,
        db_column="ENTITY_CD",
        blank=True,
        verbose_name="entity code",
        help_text="Entity Code describing the filer",
        choices=ENTITY_CODE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712034", start_page=82),
            DocumentCloud(id="2712033", start_page=68),
        ],
    )
    filer_id = fields.CharField(
        verbose_name="filer ID",
        db_column="FILER_ID",
        max_length=9,
        blank=True,
        db_index=True,
        help_text="Filer's unique identification number",
    )
    filer_namf = fields.CharField(
        max_length=45,
        db_column="FILER_NAMF",
        blank=True,
        help_text="Filer first name",
    )
    filer_naml = fields.CharField(
        max_length=200,
        db_column="FILER_NAML",
        blank=True,
        help_text="Filer last name",
    )
    filer_nams = fields.CharField(
        max_length=10,
        db_column="FILER_NAMS",
        blank=True,
        help_text="Filer suffix",
    )
    filer_namt = fields.CharField(
        max_length=10,
        db_column="FILER_NAMT",
        blank=True,
        help_text="Filer title or prefix",
    )
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        verbose_name="filing ID",
        help_text="Unique filing identification number",
        null=True,
        blank=True,
    )
    firm_name = fields.CharField(
        max_length=200,
        db_column="FIRM_NAME",
        blank=True,
        help_text="Name of the lobbyist employer or firm. Applies to \
Forms 604, 606, 607.",
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        max_length=4,
        db_column="FORM_TYPE",
        db_index=True,
        help_text="Name of the source filing form or schedule",
        choices=FORM_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=68),
            DocumentCloud(id="2712034", start_page=82),
        ],
    )
    ind_cb = fields.CharField(
        max_length=1,
        db_column="IND_CB",
        blank=True,
        help_text="Individual checkbox",
    )
    IND_CLASS_CHOICES = (
        ("AGR", "Agriculture"),
        ("EDU", "Education"),
        ("GOV", "Government"),
        ("HEA", "Health"),
        ("LAB", "Labor Unions"),
        ("LEG", "Legal"),
        ("OTH", "Other"),
        ("POL", "Political Organizations"),
        ("PUB", "Public Employees"),
        ("UTL", "Utilities"),
    )
    ind_class = fields.CharField(
        max_length=3,
        db_column="IND_CLASS",
        blank=True,
        choices=IND_CLASS_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712034", start_page=85),
            DocumentCloud(id="2712033", start_page=70),
        ],
        help_text="Classification values to category industry related \
entities. This field is exclusive of the business class field. One these \
must be populated but not both.",
    )
    ind_descr = fields.CharField(
        max_length=100,
        db_column="IND_DESCR",
        blank=True,
        help_text="Description of industry classification \
if coded as other",
    )
    INFLUEN_YN_CHOICES = (
        ("Y", "Yes"),
        ("y", "Yes"),
        ("N", "No"),
        ("n", "No"),
        ("X", "Yes"),
    )
    influen_yn = fields.CharField(
        null=True,
        max_length=1,
        choices=INFLUEN_YN_CHOICES,
        db_column="INFLUEN_YN",
        blank=True,
        help_text="Attempt to influence state legislation",
        documentcloud_pages=[
            DocumentCloud(id="2712034", start_page=86),
            DocumentCloud(id="2712033", start_page=71),
        ],
    )
    l_firm_cb = fields.CharField(
        max_length=1,
        db_column="L_FIRM_CB",
        blank=True,
        help_text="'Lobbying firm within the ... ' checkbox. \
Applies to Form 607.",
    )
    lby_604_cb = fields.CharField(
        max_length=1,
        db_column="LBY_604_CB",
        blank=True,
        help_text="'Lobbying Agency in this 604 statement' checkbox. \
Applies to Form 604.",
    )
    lby_reg_cb = fields.CharField(
        max_length=1,
        db_column="LBY_REG_CB",
        blank=True,
        help_text="'Lobbying Agency in form 601/603 registration \
statement' checkbox. Applies to Form 604.",
    )
    lobby_cb = fields.CharField(
        max_length=1,
        db_column="LOBBY_CB",
        blank=True,
        help_text="'Lobbying within the meaning...' checkbox. \
Applies to Form 607.",
    )
    lobby_int = fields.CharField(
        max_length=300,
        db_column="LOBBY_INT",
        blank=True,
        help_text="Description of Part III lobbying interests. \
Applies to Form 603",
    )
    ls_beg_yr = fields.CharField(
        null=True,
        db_column="LS_BEG_YR",
        blank=True,
        max_length=5,
        help_text="Year legislative session begins",
    )
    ls_end_yr = fields.CharField(
        db_column="LS_END_YR",
        max_length=5,
        help_text="Year legislative sessions ends",
    )
    # mail_adr1 = fields.CharField(
    #   max_length=55,
    #   db_column='MAIL_ADR1',
    #   blank=True,
    #   help_text='First line of filer mailing street address',
    # )
    # mail_adr2 = fields.CharField(
    #   max_length=55,
    #   db_column='MAIL_ADR2',
    #   blank=True,
    #   help_text='Second line of filer mailing street address',
    # )
    mail_city = fields.CharField(
        max_length=30,
        db_column="MAIL_CITY",
        blank=True,
        help_text="Filer mailing address city",
    )
    mail_phon = fields.CharField(
        max_length=20,
        db_column="MAIL_PHON",
        blank=True,
        help_text="Filer mailing address phone number",
    )
    mail_st = fields.CharField(
        max_length=2,
        db_column="MAIL_ST",
        blank=True,
        help_text="Filer mailing address state",
    )
    mail_zip4 = fields.CharField(
        max_length=10,
        db_column="MAIL_ZIP4",
        blank=True,
        help_text="Filer mailing address ZIP Code",
    )
    newcert_cb = fields.CharField(
        max_length=1,
        db_column="NEWCERT_CB",
        blank=True,
        help_text="Will require a new certification checkbox. \
Applies to Form 604.",
    )
    oth_cb = fields.CharField(
        max_length=1,
        db_column="OTH_CB",
        blank=True,
        help_text="Other checkbox",
    )
    prn_namf = fields.CharField(
        max_length=45,
        db_column="PRN_NAMF",
        blank=True,
        help_text="Signer first name",
    )
    prn_naml = fields.CharField(
        max_length=200,
        db_column="PRN_NAML",
        blank=True,
        help_text="Signer last name",
    )
    prn_nams = fields.CharField(
        max_length=10,
        db_column="PRN_NAMS",
        blank=True,
        help_text="Signer suffix",
    )
    prn_namt = fields.CharField(
        max_length=10,
        db_column="PRN_NAMT",
        blank=True,
        help_text="Signer title or prefix",
    )
    qual_date = fields.DateField(
        null=True,
        db_column="QUAL_DATE",
        blank=True,
        help_text="Date qualified. Applies to forms 601 and 603. Only \
occurs once in lobbying filings.",
    )
    REC_TYPE_CHOICES = (("CVR", "CVR"),)
    rec_type = fields.CharField(
        verbose_name="record type",
        db_column="REC_TYPE",
        max_length=4,
        db_index=True,
        choices=REC_TYPE_CHOICES,
        help_text="Record Type Value: CVR",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=68),
            DocumentCloud(id="2712034", start_page=82),
        ],
    )
    rencert_cb = fields.CharField(
        max_length=1,
        db_column="RENCERT_CB",
        blank=True,
        help_text="Will take a renewel certification checkbox. Applies to \
Form 604.",
    )
    report_num = fields.CharField(
        max_length=3,
        db_column="REPORT_NUM",
        blank=True,
        help_text="Amendment number as reported by the filer. 000 is the \
original. 001-999 are amendments.",
    )
    rpt_date = fields.DateField(
        db_column="RPT_DATE",
        null=True,
        help_text="Date this report or amendment is filed, as reported \
by the filer",
    )
    sender_id = fields.CharField(
        max_length=9,
        db_column="SENDER_ID",
        help_text="Identification number of the lobbyist entity submitting \
this report. This is equal to the filer ID if the filer is the submitting \
the report and the firm or employer if they are submitting the report.",
    )
    sig_date = fields.DateField(
        null=True, db_column="SIG_DATE", blank=True, help_text="Date signed"
    )
    sig_loc = fields.CharField(
        max_length=45,
        db_column="SIG_LOC",
        blank=True,
        help_text="Signer city and state",
    )
    sig_namf = fields.CharField(
        max_length=45,
        db_column="SIG_NAMF",
        blank=True,
        help_text="Signer first name",
    )
    sig_naml = fields.CharField(
        max_length=200,
        db_column="SIG_NAML",
        blank=True,
        help_text="Signer last name",
    )
    sig_nams = fields.CharField(
        max_length=10,
        db_column="SIG_NAMS",
        blank=True,
        help_text="Signer suffix",
    )
    sig_namt = fields.CharField(
        max_length=10,
        db_column="SIG_NAMT",
        blank=True,
        help_text="Signer title or prefix",
    )
    sig_title = fields.CharField(
        max_length=45,
        db_column="SIG_TITLE",
        blank=True,
        help_text="Title of signer",
    )
    st_agency = fields.CharField(
        max_length=100,
        db_column="ST_AGENCY",
        blank=True,
        help_text="List of identified state agencies. Applies to Form 604.",
    )
    ST_LEG_YN_CHOICES = (
        ("Y", "Yes"),
        ("y", "Yes"),
        ("N", "No"),
        ("n", "No"),
        ("X", "Yes"),
        ("x", "Yes"),
    )
    st_leg_yn = fields.CharField(
        max_length=1,
        db_column="ST_LEG_YN",
        choices=ST_LEG_YN_CHOICES,
        blank=True,
        help_text="Will lobby state legislature checkbox. \
Applies to Form 604.",
        documentcloud_pages=[
            DocumentCloud(id="2712034", start_page=86),
            DocumentCloud(id="2712033", start_page=71),
        ],
    )
    stmt_firm = fields.CharField(
        max_length=90,
        db_column="STMT_FIRM",
        blank=True,
        help_text="Lobby firm named in 'Statement of Responsible Officer'\
This field only applies to Form 601.",
    )
    trade_cb = fields.CharField(
        max_length=1,
        db_column="TRADE_CB",
        null=True,
        blank=True,
        help_text="Industry, trade or professional checkbox",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "CVR_REGISTRATION_CD"
        ordering = ("-rpt_date",)

    def __str__(self):
        return str(self.filing_id)


class Cvr2RegistrationCd(CalAccessBaseModel):
    """
    Additional names provided on lobbying-registration forms.
    """

    UNIQUE_KEY = ("FILING_ID", "AMEND_ID", "LINE_ITEM", "REC_TYPE", "FORM_TYPE")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=8),
        DocumentCloud(id="2711614", start_page=44, end_page=45),
        DocumentCloud(id="2711616", start_page=37, end_page=37),
        DocumentCloud(id="2712033", start_page=72, end_page=73),
        DocumentCloud(id="2712034", start_page=87, end_page=88),
    ]
    FILING_FORMS = [
        annotations.get_form("F601"),
        annotations.get_form("F602"),
        annotations.get_form("F603"),
    ]
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        verbose_name="filing ID",
        help_text="Unique filing identification number",
    )
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment identification number. A number of 0 is the \
original filing and 1 to 999 amendments.",
        verbose_name="amendment ID",
    )
    line_item = fields.IntegerField(
        db_column="LINE_ITEM",
        help_text="Line item number of this record",
        db_index=True,
    )
    REC_TYPE_CHOICES = (("CVR2", "CVR2"),)
    rec_type = fields.CharField(
        verbose_name="record type",
        max_length=4,
        db_column="REC_TYPE",
        choices=REC_TYPE_CHOICES,
        help_text="Record Type Value: CVR2",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=72),
            DocumentCloud(id="2712034", start_page=87),
        ],
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        max_length=10,
        db_column="FORM_TYPE",
        db_index=True,
        help_text="Name of the source filing form or schedule",
        choices=FORM_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=72),
            DocumentCloud(id="2712034", start_page=87),
        ],
    )
    tran_id = fields.CharField(
        verbose_name="transaction ID",
        max_length=20,
        db_column="TRAN_ID",
        blank=True,
        help_text="Permanent value unique to this item",
    )
    ENTITY_CODE_CHOICES = (
        ("AGY", annotations.choices.LOBBYING_ENTITY_CODES["AGY"]),
        ("EMP", annotations.choices.LOBBYING_ENTITY_CODES["EMP"]),
        ("FRM", annotations.choices.LOBBYING_ENTITY_CODES["FRM"]),
        ("LBY", annotations.choices.LOBBYING_ENTITY_CODES["LBY"]),
        ("MBR", annotations.choices.LOBBYING_ENTITY_CODES["MBR"]),
        ("SCL", annotations.choices.LOBBYING_ENTITY_CODES["SCL"]),
    )
    entity_cd = fields.CharField(
        max_length=3,
        db_column="ENTITY_CD",
        blank=True,
        verbose_name="entity code",
        choices=ENTITY_CODE_CHOICES,
        help_text="Entity code of the entity described by the record",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=72),
            DocumentCloud(id="2712034", start_page=87),
        ],
    )
    entity_id = fields.CharField(
        verbose_name="Entity ID",
        max_length=9,
        db_column="ENTITY_ID",
        blank=True,
        help_text="Identification number of the entity described by \
the record",
    )
    enty_naml = fields.CharField(
        max_length=200,
        db_column="ENTY_NAML",
        blank=True,
        help_text="Entity last name",
    )
    enty_namf = fields.CharField(
        max_length=45,
        db_column="ENTY_NAMF",
        blank=True,
        help_text="Entity first name",
    )
    enty_namt = fields.CharField(
        max_length=10,
        db_column="ENTY_NAMT",
        blank=True,
        help_text="Entity title or suffix",
    )
    enty_nams = fields.CharField(
        max_length=10,
        db_column="ENTY_NAMS",
        blank=True,
        help_text="Entity suffix",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "CVR2_REGISTRATION_CD"

    def __str__(self):
        return str(self.filing_id)


class LobbyAmendmentsCd(CalAccessBaseModel):
    """
    Amendments to lobbyist-registration forms.
    """

    UNIQUE_KEY = ("FILING_ID", "AMEND_ID", "REC_TYPE", "FORM_TYPE")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=10),
        DocumentCloud(id="2711614", start_page=90, end_page=91),
        DocumentCloud(id="2711616", start_page=64, end_page=66),
        DocumentCloud(id="2712033", start_page=74),
        DocumentCloud(id="2712034", start_page=88, end_page=89),
    ]
    FILING_FORMS = [
        annotations.get_form("F601"),
        annotations.get_form("F603"),
    ]
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        verbose_name="filing ID",
        help_text="Unique filing identification number",
    )
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment identification number. A number of 0 is the \
original filing and 1 to 999 amendments.",
        verbose_name="amendment ID",
    )
    REC_TYPE_CHOICES = (("F605", "F605"),)
    rec_type = fields.CharField(
        verbose_name="record type",
        db_column="REC_TYPE",
        max_length=4,
        db_index=True,
        choices=REC_TYPE_CHOICES,
        help_text="Record Type Value: F605",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=74),
            DocumentCloud(id="2712034", start_page=88),
        ],
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        max_length=9,
        db_column="FORM_TYPE",
        help_text="Name of the source filing form or schedule",
        db_index=True,
        choices=FORM_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=74),
            DocumentCloud(id="2712034", start_page=88),
        ],
    )
    exec_date = fields.DateField(
        max_length=22,
        db_column="EXEC_DATE",
        verbose_name="executed date",
        help_text="Date this amendment executed on",
    )
    from_date = fields.DateField(
        max_length=22,
        db_column="FROM_DATE",
        verbose_name="reported from date",
        help_text="Reporting period from date of original report",
    )
    thru_date = fields.DateField(
        max_length=22,
        db_column="THRU_DATE",
        verbose_name="reported through date",
        help_text="Reporting date to/through date of original",
    )
    add_l_cb = fields.CharField(
        max_length=1,
        db_column="ADD_L_CB",
        blank=True,
        help_text="Add lobbyist checkbox",
    )
    add_l_eff = fields.DateField(
        null=True,
        db_column="ADD_L_EFF",
        blank=True,
        help_text="Add lobbyist effective date",
    )
    a_l_naml = fields.CharField(
        max_length=200,
        db_column="A_L_NAML",
        blank=True,
        help_text="Add lobbyist last name",
    )
    a_l_namf = fields.CharField(
        max_length=45,
        db_column="A_L_NAMF",
        blank=True,
        help_text="Add lobbyist first name",
    )
    a_l_namt = fields.CharField(
        max_length=10,
        db_column="A_L_NAMT",
        blank=True,
        help_text="Add lobbyist title or prefix",
    )
    a_l_nams = fields.CharField(
        max_length=10,
        db_column="A_L_NAMS",
        blank=True,
        help_text="Add lobbyist suffix",
    )
    del_l_cb = fields.CharField(
        max_length=8,
        db_column="DEL_L_CB",
        blank=True,
        help_text="Delete lobbyist checkbox",
    )
    del_l_eff = fields.CharField(
        max_length=22,
        db_column="DEL_L_EFF",
        blank=True,
        help_text="Delete lobbyist effective date",
    )
    d_l_naml = fields.CharField(
        max_length=200,
        db_column="D_L_NAML",
        blank=True,
        help_text="Delete lobbyist last name",
    )
    d_l_namf = fields.CharField(
        max_length=45,
        db_column="D_L_NAMF",
        blank=True,
        help_text="Delete lobbyist first name",
    )
    d_l_namt = fields.CharField(
        max_length=10,
        db_column="D_L_NAMT",
        blank=True,
        help_text="Delete lobbyist title or prefix",
    )
    d_l_nams = fields.CharField(
        max_length=10,
        db_column="D_L_NAMS",
        blank=True,
        help_text="Delete lobbyiest suffix",
    )
    add_le_cb = fields.CharField(
        max_length=1,
        db_column="ADD_LE_CB",
        blank=True,
        help_text="Add lobbyiest employer checkbox",
    )
    add_le_eff = fields.DateField(
        null=True,
        db_column="ADD_LE_EFF",
        blank=True,
        help_text="Add lobbyist employer effective date",
    )
    a_le_naml = fields.CharField(
        max_length=200,
        db_column="A_LE_NAML",
        blank=True,
        help_text="Add lobbyist employer last name",
    )
    a_le_namf = fields.CharField(
        max_length=45,
        db_column="A_LE_NAMF",
        blank=True,
        help_text="Add lobbyist or employer first name",
    )
    a_le_namt = fields.CharField(
        max_length=10,
        db_column="A_LE_NAMT",
        blank=True,
        help_text="Add lobbyist employer title or prefix",
    )
    a_le_nams = fields.CharField(
        max_length=10,
        db_column="A_LE_NAMS",
        blank=True,
        help_text="Add lobbyist employer suffix",
    )
    del_le_cb = fields.CharField(
        max_length=9,
        db_column="DEL_LE_CB",
        blank=True,
        help_text="Delete lobbyist employer check box",
    )
    del_le_eff = fields.CharField(
        max_length=22,
        db_column="DEL_LE_EFF",
        blank=True,
        help_text="Delete lobbyist employer effective date",
    )
    d_le_naml = fields.CharField(
        max_length=200,
        db_column="D_LE_NAML",
        blank=True,
        help_text="Delete lobbyist employer last name",
    )
    d_le_namf = fields.CharField(
        max_length=45,
        db_column="D_LE_NAMF",
        blank=True,
        help_text="Delete lobbyiest employer first name",
    )
    d_le_namt = fields.CharField(
        max_length=12,
        db_column="D_LE_NAMT",
        blank=True,
        help_text="Delete lobbyist employer name title or prefix",
    )
    d_le_nams = fields.CharField(
        max_length=10,
        db_column="D_LE_NAMS",
        blank=True,
        help_text="Delete lobbyist employer name",
    )
    add_lf_cb = fields.CharField(
        max_length=1,
        db_column="ADD_LF_CB",
        blank=True,
        help_text="Add lobbying firm checkbox",
    )
    add_lf_eff = fields.DateField(
        null=True,
        db_column="ADD_LF_EFF",
        blank=True,
        help_text="Add lobbying firm effective date",
    )
    a_lf_name = fields.CharField(
        max_length=200,
        db_column="A_LF_NAME",
        blank=True,
        help_text="Add lobbying firm name",
    )
    del_lf_cb = fields.CharField(
        max_length=1,
        db_column="DEL_LF_CB",
        blank=True,
        help_text="Delete lobbying firm checkbox",
    )
    del_lf_eff = fields.DateField(
        null=True,
        db_column="DEL_LF_EFF",
        blank=True,
        help_text="Delete lobbying firm effective date",
    )
    d_lf_name = fields.CharField(
        max_length=200,
        db_column="D_LF_NAME",
        blank=True,
        help_text="Delete lobbying firm name",
    )
    other_cb = fields.CharField(
        max_length=1,
        db_column="OTHER_CB",
        blank=True,
        help_text="Other amendments checkbox",
    )
    other_eff = fields.DateField(
        null=True,
        db_column="OTHER_EFF",
        blank=True,
        help_text="Other amendments effective date",
    )
    other_desc = fields.CharField(
        max_length=100,
        db_column="OTHER_DESC",
        blank=True,
        help_text="Description of changes",
    )
    f606_yes = fields.CharField(
        max_length=1,
        db_column="F606_YES",
        blank=True,
        help_text="Lobbyist ceasing all activity",
    )
    f606_no = fields.CharField(
        max_length=1,
        db_column="F606_NO",
        blank=True,
        help_text="Lobbyist ceasing employment but staying active",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "LOBBY_AMENDMENTS_CD"
        ordering = ("-exec_date",)

    def __str__(self):
        return str(self.filing_id)


class LobbyingChgLogCd(CalAccessBaseModel):
    """
    Log of lobbyist filings compiled for display on the state website.
    """

    UNIQUE_KEY = ("FILER_ID", "CHANGE_NO")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=10),
        DocumentCloud(id="2711614", start_page=91, end_page=92),
    ]
    filer_id = fields.IntegerField(
        verbose_name="filer ID",
        db_column="FILER_ID",
        null=True,
        db_index=True,
        help_text="Filer's unique identification number",
    )
    change_no = fields.IntegerField(
        db_column="CHANGE_NO", help_text="Number of changes this session"
    )
    session_id = fields.IntegerField(
        verbose_name="session ID",
        db_column="SESSION_ID",
        help_text="Legislative session identification number",
        null=True,
    )
    log_dt = fields.DateField(
        db_column="LOG_DT", null=True, help_text="This field is undocumented"
    )
    filer_type = fields.ForeignKeyField(
        "FilerTypesCd",
        related_name="lobby_change_logs",
        db_constraint=False,
        help_text="Foreign key referencing FilerTypesCd.filer_type",
        db_column="FILER_TYPE",
        on_delete=models.CASCADE,
    )
    correction_flag = fields.CharField(
        max_length=200,
        db_column="CORRECTION_FLG",
        help_text="This field is undocumented",
    )
    action = fields.CharField(
        max_length=200, db_column="ACTION", help_text="This field is undocumented"
    )
    attribute_changed = fields.CharField(
        max_length=200,
        db_column="ATTRIBUTE_CHANGED",
        help_text="This field is undocumented",
    )
    ethics_dt = fields.DateField(
        db_column="ETHICS_DT", null=True, help_text="This field is undocumented"
    )
    interests = fields.CharField(
        max_length=200, db_column="INTERESTS", help_text="This field is undocumented"
    )
    filer_full_name = fields.CharField(
        max_length=200,
        db_column="FILER_FULL_NAME",
        help_text="Full name of filer",
    )
    filer_city = fields.CharField(
        max_length=200,
        db_column="FILER_CITY",
        help_text="City address of filer",
    )
    filer_st = fields.CharField(
        max_length=200,
        db_column="FILER_ST",
        verbose_name="Filer state",
        help_text="State address of filer",
    )
    filer_zip = fields.IntegerField(
        db_column="FILER_ZIP",
        null=True,
        verbose_name="Filer ZIP Code",
        help_text="ZIP Code of filer",
    )
    filer_phone = fields.CharField(
        db_column="FILER_PHONE",
        null=True,
        max_length=12,
        verbose_name="Filer phone number",
        help_text="Phone number of filer",
    )
    ENTITY_TYPE_CHOICES = (
        (0, "n/a"),
        (1, "Client"),
        (2, "Employer"),
        (3, "Firm"),
        (4, "Lobbyist"),
        (10, "Major Donor"),
        (16, "Recipient Committee"),
        (20, "Treasurer/Responsible Officer"),
    )
    entity_type = fields.IntegerField(
        db_column="ENTITY_TYPE",
        null=True,
        choices=ENTITY_TYPE_CHOICES,
        help_text="This field is undocumented. The values might refer to either \
FILER_TYPES_CD.FILER_TYPE or GROUP_TYPES_CD.GRP_ID, but that's just a guess.",
    )
    entity_name = fields.CharField(
        max_length=500, db_column="ENTITY_NAME", help_text="This field is undocumented"
    )
    entity_city = fields.CharField(
        max_length=500, db_column="ENTITY_CITY", help_text="This field is undocumented"
    )
    entity_st = fields.CharField(
        max_length=500, db_column="ENTITY_ST", help_text="This field is undocumented"
    )
    entity_zip = fields.CharField(
        db_column="ENTITY_ZIP",
        blank=True,
        max_length=10,
        help_text="This field is undocumented",
    )
    entity_phone = fields.CharField(
        db_column="ENTITY_PHONE",
        null=True,
        max_length=12,
        help_text="Entity phone number",
    )
    entity_id = fields.IntegerField(
        db_column="ENTITY_ID", null=True, help_text="Entity identification number"
    )
    responsible_officer = fields.CharField(
        max_length=500,
        db_column="RESPONSIBLE_OFFICER",
        help_text="This field is undocumented",
    )
    effect_dt = fields.DateField(
        db_column="EFFECT_DT",
        null=True,
        verbose_name="Effective date",
        help_text="This field is undocumented",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "LOBBYING_CHG_LOG_CD"
        ordering = ("-log_dt",)

    def __str__(self):
        return str(self.filer_id)


class LempCd(CalAccessBaseModel):
    """
    Lobbyist employers and subcontracted clients.
    """

    UNIQUE_KEY = ("FILING_ID", "AMEND_ID", "LINE_ITEM", "REC_TYPE", "FORM_TYPE")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=10),
        DocumentCloud(id="2711614", start_page=85, end_page=86),
        DocumentCloud(id="2711616", start_page=56, end_page=57),
        DocumentCloud(id="2712033", start_page=75),
        DocumentCloud(id="2712034", start_page=90, end_page=91),
    ]
    FILING_FORMS = [
        annotations.get_form("F601").get_section("P2A"),
        annotations.get_form("F601").get_section("P2B"),
    ]
    agencylist = fields.CharField(
        max_length=200,
        db_column="AGENCYLIST",
        blank=True,
        help_text="Agencies to be lobbied",
    )
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment identification number. A number of 0 is the \
original filing and 1 to 999 amendments.",
        verbose_name="amendment ID",
    )
    # cli_adr1 = fields.CharField(
    #     max_length=55, db_column='CLI_ADR1', blank=True
    # )
    # cli_adr2 = fields.CharField(
    #     max_length=55, db_column='CLI_ADR2', blank=True
    # )
    cli_city = fields.CharField(
        max_length=30, db_column="CLI_CITY", help_text="Employing client city"
    )
    cli_namf = fields.CharField(
        max_length=45,
        db_column="CLI_NAMF",
        blank=True,
        help_text="Employing client first name",
    )
    cli_naml = fields.CharField(
        max_length=200, db_column="CLI_NAML", help_text="Employing client last name"
    )
    cli_nams = fields.CharField(
        max_length=10,
        db_column="CLI_NAMS",
        blank=True,
        help_text="Employing client suffix",
    )
    cli_namt = fields.CharField(
        max_length=10,
        db_column="CLI_NAMT",
        blank=True,
        help_text="Employing client prefix or title",
    )
    cli_phon = fields.CharField(
        max_length=20,
        db_column="CLI_PHON",
        blank=True,
        help_text="Employing client phone number",
    )
    cli_st = fields.CharField(
        max_length=2, db_column="CLI_ST", blank=True, help_text="Employing client state"
    )
    cli_zip4 = fields.CharField(
        max_length=10, db_column="CLI_ZIP4", help_text="Employing client ZIP Code"
    )
    client_id = fields.CharField(
        max_length=9,
        db_column="CLIENT_ID",
        blank=True,
        help_text="Identification number of the Part 2A employer or \
Part 2B Client/Employer",
    )
    con_period = fields.CharField(
        max_length=30,
        db_column="CON_PERIOD",
        blank=True,
        help_text="Period of the contract",
    )
    descrip = fields.CharField(
        max_length=100,
        db_column="DESCRIP",
        blank=True,
        help_text="Description of employer/client lobbying interest",
    )
    eff_date = fields.DateField(
        null=True,
        db_column="EFF_DATE",
        blank=True,
        help_text="Effective Date of Lobbying Contract",
    )
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        verbose_name="filing ID",
        help_text="Unique filing identification number",
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        max_length=7,
        db_column="FORM_TYPE",
        verbose_name="form type",
        help_text="Name of the source filing form or schedule",
        db_index=True,
        choices=FORM_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=75),
            DocumentCloud(id="2712034", start_page=90),
        ],
    )
    line_item = fields.IntegerField(
        db_column="LINE_ITEM",
        help_text="Line item number of this record",
        db_index=True,
    )
    REC_TYPE_CHOICES = (("LEMP", "LEMP"),)
    rec_type = fields.CharField(
        verbose_name="record type",
        db_column="REC_TYPE",
        max_length=4,
        db_index=True,
        help_text="Record Type Value: LEMP",
        choices=REC_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=75),
            DocumentCloud(id="2712034", start_page=90),
        ],
    )
    # sub_adr1 = fields.CharField(
    #     max_length=55, db_column='SUB_ADR1', blank=True
    # )
    # sub_adr2 = fields.CharField(
    #     max_length=55, db_column='SUB_ADR2', blank=True
    # )
    sub_city = fields.CharField(
        max_length=30,
        db_column="SUB_CITY",
        blank=True,
        help_text="Subcontracting lobbying firm city",
    )
    sub_name = fields.CharField(
        max_length=200,
        db_column="SUB_NAME",
        blank=True,
        help_text="Subcontracting lobbying firms name",
    )
    sub_phon = fields.CharField(
        max_length=20,
        db_column="SUB_PHON",
        blank=True,
        help_text="Subcontracting lobbying firm phone number",
    )
    sub_st = fields.CharField(
        max_length=2,
        db_column="SUB_ST",
        blank=True,
        help_text="Subcontracting lobbying firm state",
    )
    sub_zip4 = fields.CharField(
        max_length=10,
        db_column="SUB_ZIP4",
        blank=True,
        help_text="Subcontracting lobbying firm ZIP Code",
    )
    subfirm_id = fields.CharField(
        max_length=9,
        db_column="SUBFIRM_ID",
        blank=True,
        help_text="Identification number of subcontracting lobbying firm",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "LEMP_CD"
        ordering = ("-eff_date",)

    def __str__(self):
        return str(self.filing_id)


class CvrLobbyDisclosureCd(CalAccessBaseModel):
    """
    The cover page of lobbying-disclosure forms.
    """

    UNIQUE_KEY = ("FILING_ID", "AMEND_ID", "REC_TYPE", "FORM_TYPE")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=8),
        DocumentCloud(id="2711614", start_page=32, end_page=35),
        DocumentCloud(id="2711616", start_page=17, end_page=21),
        DocumentCloud(id="2712033", start_page=53, end_page=56),
        DocumentCloud(id="2712034", start_page=66, end_page=70),
    ]
    FILING_FORMS = [
        annotations.get_form("F615"),
        annotations.get_form("F625"),
        annotations.get_form("F635"),
        annotations.get_form("F645"),
    ]
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment identification number. A number of 0 is the \
original filing and 1 to 999 amendments.",
        verbose_name="amendment ID",
    )
    ctrib_n_cb = fields.CharField(
        max_length=1,
        db_column="CTRIB_N_CB",
        blank=True,
        help_text="'Campaign contribtions? P4 attached' checkbox. Applies \
to forms 625, 635, 645.",
    )
    ctrib_y_cb = fields.CharField(
        max_length=1,
        db_column="CTRIB_Y_CB",
        blank=True,
        help_text="'Campaign contribtions? P4 attached' checkbox. Applies \
to forms 625, 635, 645.",
    )
    cum_beg_dt = fields.DateField(
        null=True,
        db_column="CUM_BEG_DT",
        blank=True,
        help_text="Cumulative period beginning date",
    )
    ENTITY_CODE_CHOICES = (
        ("CLI", "Unknown"),
        ("FRM", annotations.choices.LOBBYING_ENTITY_CODES["FRM"]),
        ("IND", annotations.choices.LOBBYING_ENTITY_CODES["IND"]),
        ("LBY", annotations.choices.LOBBYING_ENTITY_CODES["LBY"]),
        ("LCO", annotations.choices.LOBBYING_ENTITY_CODES["LCO"]),
        ("LEM", annotations.choices.LOBBYING_ENTITY_CODES["LEM"]),
        ("OTH", annotations.choices.LOBBYING_ENTITY_CODES["OTH"]),
    )
    entity_cd = fields.CharField(
        max_length=3,
        db_column="ENTITY_CD",
        blank=True,
        verbose_name="entity code",
        choices=ENTITY_CODE_CHOICES,
        help_text="Entity Code describing the filer",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=53),
            DocumentCloud(id="2712034", start_page=67),
        ],
    )
    filer_id = fields.CharField(
        verbose_name="filer ID",
        db_column="FILER_ID",
        max_length=9,
        blank=True,
        db_index=True,
        help_text="Filer's unique identification number",
    )
    filer_namf = fields.CharField(
        max_length=45,
        db_column="FILER_NAMF",
        blank=True,
        help_text="Filer first name",
    )
    filer_naml = fields.CharField(
        max_length=200,
        db_column="FILER_NAML",
        help_text="Filer last name or business name",
    )
    filer_nams = fields.CharField(
        max_length=10,
        db_column="FILER_NAMS",
        blank=True,
        help_text="Filer suffix",
    )
    filer_namt = fields.CharField(
        max_length=10,
        db_column="FILER_NAMT",
        blank=True,
        help_text="Filer title or prefix",
    )
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        verbose_name="filing ID",
        help_text="Unique filing identification number",
    )
    # firm_adr1 = fields.CharField(
    #   max_length=55,
    #   db_column='FIRM_ADR1',
    #   blank=True,
    #   help_text='First line of Firm, employer or coalition address',',
    # )
    # firm_adr2 = fields.CharField(
    #   max_length=55,
    #   db_column='FIRM_ADR2',
    #   blank=True,
    #   help_text='Second line of firm, employer or coalition address',
    # )
    firm_city = fields.CharField(
        max_length=30,
        db_column="FIRM_CITY",
        blank=True,
        help_text="Firm, employer or coalition business city",
    )
    firm_id = fields.CharField(
        max_length=9,
        db_column="FIRM_ID",
        blank=True,
        help_text="Identification number of firm, employer or coalition",
    )
    firm_name = fields.CharField(
        max_length=200,
        db_column="FIRM_NAME",
        blank=True,
        help_text="Name of firm, employer or coalition",
    )
    firm_phon = fields.CharField(
        max_length=20,
        db_column="FIRM_PHON",
        blank=True,
        help_text="Firm, employer or coalition business phone number",
    )
    firm_st = fields.CharField(
        max_length=2,
        db_column="FIRM_ST",
        blank=True,
        help_text="Firm, employer or coalition business state",
    )
    firm_zip4 = fields.CharField(
        max_length=10,
        db_column="FIRM_ZIP4",
        blank=True,
        help_text="Form, employer or coalition business ZIP Code",
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        max_length=4,
        db_column="FORM_TYPE",
        help_text="Name of the source filing form or schedule",
        db_index=True,
        choices=FORM_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=53),
            DocumentCloud(id="2712034", start_page=66),
        ],
    )
    from_date = fields.DateField(
        db_column="FROM_DATE",
        null=True,
        help_text="Reporting period from date",
    )
    lby_actvty = fields.CharField(
        max_length=400,
        db_column="LBY_ACTVTY",
        blank=True,
        help_text="Description of lobbying activity. Applies to forms \
635 and 645. Additional description may be provided in text records.",
    )
    lobby_n_cb = fields.CharField(
        max_length=1,
        db_column="LOBBY_N_CB",
        blank=True,
        help_text="'Lobbying activity none' checkbox. Applies \
only to Form 625.",
    )
    lobby_y_cb = fields.CharField(
        max_length=1,
        db_column="LOBBY_Y_CB",
        blank=True,
        help_text="'Lobbying activity Form 630 attached' checkbox. Applies \
only to Form 625.",
    )
    # mail_adr1 = fields.CharField(
    #   max_length=55,
    #   db_column='MAIL_ADR1',
    #   blank=True,
    #   help_text='First line of filer mailing street address',
    # )
    # mail_adr2 = fields.CharField(
    #   max_length=55,
    #   db_column='MAIL_ADR2',
    #   blank=True,
    #   help_text='Second line of filer mailing street address',
    # )
    mail_city = fields.CharField(
        max_length=30,
        db_column="MAIL_CITY",
        blank=True,
        help_text="Filer mailing address city",
    )
    mail_phon = fields.CharField(
        max_length=20,
        db_column="MAIL_PHON",
        blank=True,
        help_text="Filer mailing address phone number",
    )
    mail_st = fields.CharField(
        max_length=2,
        db_column="MAIL_ST",
        blank=True,
        help_text="Filer mailing address state",
    )
    mail_zip4 = fields.CharField(
        max_length=10,
        db_column="MAIL_ZIP4",
        blank=True,
        help_text="Filer mailing address ZIP Code",
    )
    major_namf = fields.CharField(
        max_length=45,
        db_column="MAJOR_NAMF",
        blank=True,
        help_text="Major donor first name. Applies only to \
individuals and forms 625, 635, 645.",
    )
    major_naml = fields.CharField(
        max_length=200,
        db_column="MAJOR_NAML",
        blank=True,
        help_text="Major donor last name. Applies only to \
individuals and forms 625, 635, 645.",
    )
    major_nams = fields.CharField(
        max_length=10,
        db_column="MAJOR_NAMS",
        blank=True,
        help_text="Major donor suffix. Applies only to \
individuals and forms 625, 635, 645.",
    )
    major_namt = fields.CharField(
        max_length=10,
        db_column="MAJOR_NAMT",
        blank=True,
        help_text="Major donor title or prefix. Applies only to \
individuals and forms 625, 635, 645.",
    )
    nopart1_cb = fields.CharField(
        max_length=1,
        db_column="NOPART1_CB",
        blank=True,
        help_text="'No Part I information' checkbox. Applies only \
to Form 615.",
    )
    nopart2_cb = fields.CharField(
        max_length=1,
        db_column="NOPART2_CB",
        blank=True,
        help_text="'No Part II information' checkbox. Applies only \
to Form 615.",
    )
    part1_1_cb = fields.CharField(
        max_length=1,
        db_column="PART1_1_CB",
        blank=True,
        help_text="'Partners, owners Form 615 attached ...' checkbox. \
Applies only to form 625.",
    )
    part1_2_cb = fields.CharField(
        max_length=1,
        db_column="PART1_2_CB",
        blank=True,
        help_text="'Partners, owners listed below ...' checkbox. Applies only \
to Form 625.",
    )
    prn_namf = fields.CharField(
        max_length=45,
        db_column="PRN_NAMF",
        blank=True,
        help_text="Signer first name",
    )
    prn_naml = fields.CharField(
        max_length=200,
        db_column="PRN_NAML",
        blank=True,
        help_text="Signer last name",
    )
    prn_nams = fields.CharField(
        max_length=10,
        db_column="PRN_NAMS",
        blank=True,
        help_text="Signer suffix",
    )
    prn_namt = fields.CharField(
        max_length=10,
        db_column="PRN_NAMT",
        blank=True,
        help_text="Signer title or prefix",
    )
    rcpcmte_id = fields.CharField(
        max_length=9,
        db_column="RCPCMTE_ID",
        blank=True,
        help_text="Recipient committee or major donor identification number",
    )
    rcpcmte_nm = fields.CharField(
        max_length=200,
        db_column="RCPCMTE_NM",
        blank=True,
        help_text="Recipient committee name",
    )
    REC_TYPE_CHOICES = (("CVR", "CVR"),)
    rec_type = fields.CharField(
        verbose_name="record type",
        db_column="REC_TYPE",
        help_text="Record Type Value: CVR",
        max_length=4,
        db_index=True,
        choices=REC_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=53),
            DocumentCloud(id="2712034", start_page=66),
        ],
    )
    report_num = fields.CharField(
        max_length=3,
        db_column="REPORT_NUM",
        help_text="Amendment number. 000 is the original. \
001-999 are amendments.",
    )
    rpt_date = fields.DateField(
        db_column="RPT_DATE",
        null=True,
        help_text="Date this report was filed, as reported by the filer",
    )
    sender_id = fields.CharField(
        max_length=9,
        db_column="SENDER_ID",
        help_text="Identification number of lobbyist entity that is \
submitting this report. The field is used to authenticate the filer and \
allows the firm to submit forms for its lobbyists.",
    )
    sig_date = fields.DateField(
        db_column="SIG_DATE",
        null=True,
        help_text="Date when signed",
    )
    sig_loc = fields.CharField(
        max_length=45,
        db_column="SIG_LOC",
        blank=True,
        help_text="Signer city and state",
    )
    sig_namf = fields.CharField(
        max_length=45,
        db_column="SIG_NAMF",
        blank=True,
        help_text="Signer first name",
    )
    sig_naml = fields.CharField(
        max_length=200,
        db_column="SIG_NAML",
        blank=True,
        help_text="Signer last name",
    )
    sig_nams = fields.CharField(
        max_length=10,
        db_column="SIG_NAMS",
        blank=True,
        help_text="Signer suffix",
    )
    sig_namt = fields.CharField(
        max_length=10,
        db_column="SIG_NAMT",
        blank=True,
        help_text="Signer title or prefix",
    )
    sig_title = fields.CharField(
        max_length=45,
        db_column="SIG_TITLE",
        blank=True,
        help_text="Title of signer",
    )
    thru_date = fields.DateField(
        db_column="THRU_DATE",
        null=True,
        help_text="Reporting period through date",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "CVR_LOBBY_DISCLOSURE_CD"
        ordering = ("-rpt_date",)

    def __str__(self):
        return str(self.filing_id)


class Cvr2LobbyDisclosureCd(CalAccessBaseModel):
    """
    Extra information from the cover sheets of lobbyist-disclosure form.
    """

    UNIQUE_KEY = ("FILING_ID", "AMEND_ID", "LINE_ITEM", "REC_TYPE", "FORM_TYPE")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=8),
        DocumentCloud(id="2711614", start_page=43, end_page=44),
        DocumentCloud(id="2711616", start_page=36),
        DocumentCloud(id="2712033", start_page=57),
        DocumentCloud(id="2712034", start_page=71),
    ]
    FILING_FORMS = [
        annotations.get_form("F625"),
        annotations.get_form("F635"),
    ]
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment identification number. A number of 0 is the \
original filing and 1 to 999 amendments.",
        verbose_name="amendment ID",
    )
    ENTITY_CODE_CHOICES = (
        ("EMP", annotations.choices.LOBBYING_ENTITY_CODES["EMP"]),
        ("OFF", annotations.choices.LOBBYING_ENTITY_CODES["OFF"]),
        ("OWN", annotations.choices.LOBBYING_ENTITY_CODES["OWN"]),
        ("PTN", annotations.choices.LOBBYING_ENTITY_CODES["PTN"]),
    )
    entity_cd = fields.CharField(
        max_length=3,
        db_column="ENTITY_CD",
        blank=True,
        verbose_name="entity code",
        choices=ENTITY_CODE_CHOICES,
        help_text="Entity code of the entity described by the record",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=57),
            DocumentCloud(id="2712034", start_page=71),
        ],
    )
    entity_id = fields.CharField(
        max_length=9,
        db_column="ENTITY_ID",
        blank=True,
        help_text="Entity identification number",
    )
    enty_namf = fields.CharField(
        max_length=45,
        db_column="ENTY_NAMF",
        blank=True,
        help_text="Entity first name",
    )
    enty_naml = fields.CharField(
        max_length=200,
        db_column="ENTY_NAML",
        blank=True,
        help_text="Entity last name or business name",
    )
    enty_nams = fields.CharField(
        max_length=10,
        db_column="ENTY_NAMS",
        blank=True,
        help_text="Entity suffix",
    )
    enty_namt = fields.CharField(
        max_length=10,
        db_column="ENTY_NAMT",
        blank=True,
        help_text="Entity title or prefix",
    )
    enty_title = fields.CharField(
        max_length=45,
        db_column="ENTY_TITLE",
        blank=True,
        help_text="Title of partner, owner, officer, employer if the \
entity is an individual. Only required by Form 635.",
    )
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        verbose_name="filing ID",
        help_text="Unique filing identification number",
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        max_length=4,
        db_column="FORM_TYPE",
        db_index=True,
        help_text="Name of the source filing form or schedule",
        choices=FORM_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=57),
            DocumentCloud(id="2712034", start_page=71),
        ],
    )
    line_item = fields.IntegerField(
        db_column="LINE_ITEM",
        help_text="Line item number of this record",
        db_index=True,
    )
    REC_TYPE_CHOICES = (("CVR2", "CVR2"),)
    rec_type = fields.CharField(
        verbose_name="record type",
        db_column="REC_TYPE",
        max_length=4,
        db_index=True,
        help_text="Record Type Value: CVR2",
        choices=REC_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=57),
            DocumentCloud(id="2712034", start_page=71),
        ],
    )
    tran_id = fields.CharField(
        verbose_name="transaction ID",
        max_length=20,
        db_column="TRAN_ID",
        blank=True,
        help_text="Permanent value unique to this item",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "CVR2_LOBBY_DISCLOSURE_CD"

    def __str__(self):
        return str(self.filing_id)


class F690P2Cd(CalAccessBaseModel):
    """
    Amendments to lobbying-disclosure filings.
    """

    UNIQUE_KEY = ("FILING_ID", "AMEND_ID", "LINE_ITEM", "REC_TYPE", "FORM_TYPE")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=8),
        DocumentCloud(id="2711614", start_page=59, end_page=60),
        DocumentCloud(id="2711616", start_page=50, end_page=51),
        DocumentCloud(id="2712033", start_page=58),
        DocumentCloud(id="2712034", start_page=72),
    ]
    FILING_FORMS = [
        annotations.get_form("F615"),
        annotations.get_form("F625"),
        annotations.get_form("F635"),
        annotations.get_form("F645"),
    ]
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        verbose_name="filing ID",
        help_text="Unique filing identification number",
    )
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment identification number. A number of 0 is the \
original filing and 1 to 999 amendments.",
        verbose_name="amendment ID",
    )
    line_item = fields.IntegerField(
        db_column="LINE_ITEM",
        help_text="Line item number of this record",
        db_index=True,
    )
    REC_TYPE_CHOICES = (("F690", "F690"),)
    rec_type = fields.CharField(
        verbose_name="record type",
        db_column="REC_TYPE",
        max_length=4,
        db_index=True,
        choices=REC_TYPE_CHOICES,
        help_text="Record Type Value: F690",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=58),
            DocumentCloud(id="2712034", start_page=72),
        ],
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        db_column="FORM_TYPE",
        max_length=4,
        help_text="Name of the source filing form or schedule",
        db_index=True,
        choices=FORM_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=58),
            DocumentCloud(id="2712034", start_page=72),
        ],
    )
    exec_date = fields.DateField(
        db_column="EXEC_DATE",
        null=True,
        help_text="date the original report (or prior amendment to \
the original report) was executed on.",
    )
    from_date = fields.DateField(
        db_column="FROM_DATE",
        null=True,
        help_text="reporting period from date of original report",
    )
    thru_date = fields.DateField(
        db_column="THRU_DATE",
        null=True,
        help_text="report period to/through date of original.",
    )
    chg_parts = fields.CharField(
        db_column="CHG_PARTS",
        max_length=100,
        blank=True,
        help_text="amended into affects items on part(s) text description.",
    )
    chg_sects = fields.CharField(
        db_column="CHG_SECTS",
        max_length=100,
        blank=True,
        help_text="amended into affects items on sections(s) \
text description.",
    )
    amend_txt1 = fields.CharField(
        db_column="AMEND_TXT1",
        max_length=330,
        blank=True,
        help_text="description of changes to the filing",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "F690P2_CD"
        ordering = ("-exec_date",)

    def __str__(self):
        return str(self.filing_id)


class LattCd(CalAccessBaseModel):
    """
    Attachments for lobbying payments.
    """

    UNIQUE_KEY = ("FILING_ID", "AMEND_ID", "LINE_ITEM", "REC_TYPE", "FORM_TYPE")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=81, end_page=82),
        DocumentCloud(id="2711616", start_page=52, end_page=53),
        DocumentCloud(id="2712033", start_page=65),
        DocumentCloud(id="2712034", start_page=79, end_page=80),
    ]
    FILING_FORMS = [
        annotations.get_form("S630"),
        annotations.get_form("S635C"),
        annotations.get_form("S640"),
    ]
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment identification number. A number of 0 is the \
original filing and 1 to 999 amendments.",
        verbose_name="amendment ID",
    )
    amount = fields.DecimalField(
        max_digits=16,
        decimal_places=2,
        db_column="AMOUNT",
        help_text="Amount of payment",
    )
    cum_amt = fields.DecimalField(
        max_digits=16,
        decimal_places=2,
        db_column="CUM_AMT",
        help_text="Cumulative total to date",
    )
    cumbeg_dt = fields.DateField(
        db_column="CUMBEG_DT",
        null=True,
        help_text="Cumulative period beginning to date",
    )
    ENTITY_CODE_CHOICES = (
        ("FRM", annotations.choices.LOBBYING_ENTITY_CODES["FRM"]),
        ("IND", annotations.choices.LOBBYING_ENTITY_CODES["IND"]),
        ("LBY", annotations.choices.LOBBYING_ENTITY_CODES["LBY"]),
        ("LCO", annotations.choices.LOBBYING_ENTITY_CODES["LCO"]),
        ("LEM", annotations.choices.LOBBYING_ENTITY_CODES["LEM"]),
        ("OTH", annotations.choices.LOBBYING_ENTITY_CODES["OTH"]),
        ("RCP", annotations.choices.LOBBYING_ENTITY_CODES["RCP"]),
    )
    entity_cd = fields.CharField(
        max_length=3,
        db_column="ENTITY_CD",
        blank=True,
        verbose_name="entity code",
        choices=ENTITY_CODE_CHOICES,
        help_text="Entity Code of the Payment Recipient/Payee",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=65),
            DocumentCloud(id="2712034", start_page=80),
        ],
    )
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        verbose_name="filing ID",
        help_text="Unique filing identification number",
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        max_length=6,
        db_column="FORM_TYPE",
        help_text="Name of the source filing form or schedule",
        db_index=True,
        choices=FORM_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2711616", start_page=52),
            DocumentCloud(id="2712033", start_page=65),
            DocumentCloud(id="2712034", start_page=79),
        ],
    )
    line_item = fields.IntegerField(
        db_column="LINE_ITEM",
        help_text="Line item number of this record",
        db_index=True,
    )
    memo_code = fields.CharField(
        max_length=1,
        db_column="MEMO_CODE",
        blank=True,
        help_text="Memo amount flag",
    )
    memo_refno = fields.CharField(
        max_length=20,
        db_column="MEMO_REFNO",
        blank=True,
        help_text="Reference to the text in a TEXT record",
    )
    pmt_date = fields.DateField(
        db_column="PMT_DATE",
        null=True,
        help_text="Date of payment",
    )
    REC_TYPE_CHOICES = (("LATT", "LATT"),)
    rec_type = fields.CharField(
        verbose_name="record type",
        db_column="REC_TYPE",
        max_length=4,
        db_index=True,
        help_text="Record Type Value: LATT",
        choices=REC_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=65),
            DocumentCloud(id="2712034", start_page=79),
        ],
    )
    # recip_adr1 = fields.CharField(
    #   max_length=55,
    #   db_column='RECIP_ADR1',
    #   blank=True,
    #   help_text='First line of the recipient street address',
    # )
    # recip_adr2 = fields.CharField(
    #   max_length=55,
    #   db_column='RECIP_ADR2',
    #   blank=True,
    #   help_text='Second line of the recipient street address',
    # )
    recip_city = fields.CharField(
        max_length=30,
        db_column="RECIP_CITY",
        blank=True,
        help_text="Recipient city",
    )
    recip_namf = fields.CharField(
        max_length=45,
        db_column="RECIP_NAMF",
        blank=True,
        help_text="Recipient first name",
    )
    recip_naml = fields.CharField(
        max_length=200,
        db_column="RECIP_NAML",
        blank=True,
        help_text="Recipient last name or business name",
    )
    recip_nams = fields.CharField(
        max_length=10,
        db_column="RECIP_NAMS",
        blank=True,
        help_text="Recipient suffix",
    )
    recip_namt = fields.CharField(
        max_length=10,
        db_column="RECIP_NAMT",
        blank=True,
        help_text="Recipient title or prefix",
    )
    recip_st = fields.CharField(
        max_length=2,
        db_column="RECIP_ST",
        blank=True,
        help_text="Recipient state",
    )
    recip_zip4 = fields.CharField(
        max_length=10,
        db_column="RECIP_ZIP4",
        blank=True,
        help_text="Recipient ZIP Code",
    )
    tran_id = fields.CharField(
        verbose_name="transaction ID",
        max_length=20,
        db_column="TRAN_ID",
        blank=True,
        help_text="Permanent value unique to this item",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "LATT_CD"
        ordering = ("-pmt_date",)

    def __str__(self):
        return str(self.filing_id)


class LexpCd(CalAccessBaseModel):
    """
    Lobbying activity expenditures.
    """

    UNIQUE_KEY = ("FILING_ID", "AMEND_ID", "LINE_ITEM", "REC_TYPE", "FORM_TYPE")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=10),
        DocumentCloud(id="2711614", start_page=86, end_page=87),
        DocumentCloud(id="2711616", start_page=58, end_page=59),
        DocumentCloud(id="2712033", start_page=61),
        DocumentCloud(id="2712034", start_page=74, end_page=75),
    ]
    FILING_FORMS = [
        annotations.get_form("F615").get_section("P1"),
        annotations.get_form("F625").get_section("P3A"),
        annotations.get_form("F635").get_section("P3C"),
        annotations.get_form("F645").get_section("P2A"),
    ]
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment identification number. A number of 0 is the \
original filing and 1 to 999 amendments.",
        verbose_name="amendment ID",
    )
    amount = fields.DecimalField(
        decimal_places=2,
        null=True,
        max_digits=14,
        db_column="AMOUNT",
        blank=True,
        help_text="Amount of payment",
    )
    bakref_tid = fields.CharField(
        max_length=20,
        db_column="BAKREF_TID",
        blank=True,
        help_text="Backreference to the tranaction identifer of parent record",
    )
    bene_amt = fields.CharField(
        max_length=12,
        db_column="BENE_AMT",
        blank=True,
        help_text="Amount benefiting benficiary",
    )
    bene_name = fields.CharField(
        max_length=90,
        db_column="BENE_NAME",
        blank=True,
        help_text="Name of the person beneifiting",
    )
    bene_posit = fields.CharField(
        max_length=90,
        db_column="BENE_POSIT",
        blank=True,
        help_text="Official position of the person beneifiting",
    )
    credcardco = fields.CharField(
        max_length=200,
        db_column="CREDCARDCO",
        blank=True,
        help_text="Name of the credit card company, if paid using a card",
    )
    ENTITY_CODE_CHOICES = (
        ("IND", annotations.choices.LOBBYING_ENTITY_CODES["IND"]),
        ("OTH", annotations.choices.LOBBYING_ENTITY_CODES["OTH"]),
    )
    entity_cd = fields.CharField(
        max_length=3,
        db_column="ENTITY_CD",
        blank=True,
        verbose_name="entity code",
        choices=ENTITY_CODE_CHOICES,
        help_text="Entity Code of the Payee",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=61),
            DocumentCloud(id="2712034", start_page=75),
        ],
    )
    expn_date = fields.DateField(
        null=True,
        db_column="EXPN_DATE",
        blank=True,
        help_text="Date of expenditure",
    )
    expn_dscr = fields.CharField(
        max_length=90,
        db_column="EXPN_DSCR",
        blank=True,
        help_text="Purpose of the expense and a description or explanation",
    )
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        verbose_name="filing ID",
        help_text="Unique filing identification number",
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        max_length=7,
        db_column="FORM_TYPE",
        help_text="Name of the source filing form or schedule",
        db_index=True,
        choices=FORM_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=61),
            DocumentCloud(id="2712034", start_page=74),
        ],
    )
    line_item = fields.IntegerField(
        db_column="LINE_ITEM",
        help_text="Line item number of this record",
        db_index=True,
    )
    memo_code = fields.CharField(
        max_length=1,
        db_column="MEMO_CODE",
        blank=True,
        help_text="Memo amount flag",
    )
    memo_refno = fields.CharField(
        max_length=20,
        db_column="MEMO_REFNO",
        blank=True,
        help_text="Reference to the text in a TEXT record",
    )
    # payee_adr1 = fields.CharField(
    #   max_length=55,
    #   db_column='PAYEE_ADR1',
    #   blank=True,
    #   help_text='First line of payee street address',
    # )
    # payee_adr2 = fields.CharField(
    #   max_length=55,
    #   db_column='PAYEE_ADR2',
    #   blank=True,
    #   help_text='Second line of payee street address',
    # )
    payee_city = fields.CharField(
        max_length=30,
        db_column="PAYEE_CITY",
        blank=True,
        help_text="Payee city",
    )
    payee_namf = fields.CharField(
        max_length=45,
        db_column="PAYEE_NAMF",
        blank=True,
        help_text="Payee first name",
    )
    payee_naml = fields.CharField(
        max_length=200,
        db_column="PAYEE_NAML",
        blank=True,
        help_text="Payee last name or business name",
    )
    payee_nams = fields.CharField(
        max_length=10,
        db_column="PAYEE_NAMS",
        blank=True,
        help_text="Payee suffix",
    )
    payee_namt = fields.CharField(
        max_length=10,
        db_column="PAYEE_NAMT",
        blank=True,
        help_text="Payee title or prefix",
    )
    payee_st = fields.CharField(
        max_length=2,
        db_column="PAYEE_ST",
        blank=True,
        help_text="Payee state",
    )
    payee_zip4 = fields.CharField(
        max_length=10,
        db_column="PAYEE_ZIP4",
        blank=True,
        help_text="Payee ZIP Code",
    )
    REC_TYPE_CHOICES = (("LEXP", "LEXP"),)
    rec_type = fields.CharField(
        verbose_name="record type",
        db_column="REC_TYPE",
        max_length=4,
        db_index=True,
        help_text="Record Type Value: LEXP",
        choices=REC_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=61),
            DocumentCloud(id="2712034", start_page=74),
        ],
    )
    REC_SUBTYPE_CHOICES = (
        ("1", "Main"),
        ("2", "Detail"),
    )
    recsubtype = fields.CharField(
        max_length=1,
        db_column="RECSUBTYPE",
        choices=REC_SUBTYPE_CHOICES,
        verbose_name="record subtype",
        help_text="Record Subtype",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=61),
            DocumentCloud(id="2712034", start_page=74),
        ],
    )
    tran_id = fields.CharField(
        verbose_name="transaction ID",
        max_length=20,
        db_column="TRAN_ID",
        blank=True,
        help_text="Permanent value unique to this item",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "LEXP_CD"
        ordering = ("-expn_date",)

    def __str__(self):
        return str(self.filing_id)


class LccmCd(CalAccessBaseModel):
    """
    Lobbyist campaign contributions.
    """

    UNIQUE_KEY = ("FILING_ID", "AMEND_ID", "LINE_ITEM", "REC_TYPE", "FORM_TYPE")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=10),
        DocumentCloud(id="2711614", start_page=83, end_page=84),
        DocumentCloud(id="2711616", start_page=54, end_page=55),
        DocumentCloud(id="2712033", start_page=64),
        DocumentCloud(id="2712034", start_page=78, end_page=79),
    ]
    FILING_FORMS = [
        annotations.get_form("F615").get_section("P2"),
        annotations.get_form("F625").get_section("P4B"),
        annotations.get_form("F635").get_section("P4B"),
        annotations.get_form("F645").get_section("P3B"),
    ]
    # acct_name = fields.CharField(
    #   max_length=90,
    #   db_column='ACCT_NAME',
    #   blank=True,
    #   help_text='name of separate account'
    # )
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment identification number. A number of 0 is the \
original filing and 1 to 999 amendments.",
        verbose_name="amendment ID",
    )
    amount = fields.DecimalField(
        max_digits=16,
        decimal_places=2,
        db_column="AMOUNT",
        help_text="Amount of contribution",
    )
    bakref_tid = fields.CharField(
        max_length=20,
        db_column="BAKREF_TID",
        blank=True,
        help_text="Back reference to transaction identifier of parent record",
    )
    ctrib_date = fields.DateField(
        db_column="CTRIB_DATE",
        null=True,
        help_text="Date of contribution",
    )
    ctrib_namf = fields.CharField(
        max_length=45,
        db_column="CTRIB_NAMF",
        blank=True,
        help_text="Contributor first name",
    )
    ctrib_naml = fields.CharField(
        max_length=200,
        db_column="CTRIB_NAML",
        blank=True,
        help_text="Contributor last name or business name",
    )
    ctrib_nams = fields.CharField(
        max_length=10,
        db_column="CTRIB_NAMS",
        blank=True,
        help_text="Contributor suffix",
    )
    ctrib_namt = fields.CharField(
        max_length=10,
        db_column="CTRIB_NAMT",
        blank=True,
        help_text="Contributor prefix or title.",
    )
    ENTITY_CODE_CHOICES = (
        ("COM", annotations.choices.LOBBYING_ENTITY_CODES["COM"]),
        ("RCP", annotations.choices.LOBBYING_ENTITY_CODES["RCP"]),
        # Not sure this is a valid value of lobbying records, but has over 1,000 occurrences
        ("CTL", annotations.choices.CAMPAIGN_ENTITY_CODES["CTL"]),
    )
    entity_cd = fields.CharField(
        max_length=3,
        db_column="ENTITY_CD",
        blank=True,
        verbose_name="entity code",
        help_text="Entity Code for Recipient of the Campaign Contribution Value",
        choices=ENTITY_CODE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=64),
            DocumentCloud(id="2712034", start_page=78),
        ],
    )
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        verbose_name="filing ID",
        help_text="Unique filing identification number",
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        max_length=7,
        db_column="FORM_TYPE",
        db_index=True,
        help_text="Name of the source filing form or schedule",
        choices=FORM_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=64),
            DocumentCloud(id="2712034", start_page=78, end_page=79),
        ],
    )
    line_item = fields.IntegerField(
        db_column="LINE_ITEM",
        help_text="Line item number of this record",
        db_index=True,
    )
    memo_code = fields.CharField(
        max_length=1,
        db_column="MEMO_CODE",
        blank=True,
        help_text="Memo amount flag",
    )
    memo_refno = fields.CharField(
        max_length=20,
        db_column="MEMO_REFNO",
        blank=True,
        help_text="Reference to the text contained in the TEXT record",
    )
    REC_TYPE_CHOICES = (("LCCM", "LCCM"),)
    rec_type = fields.CharField(
        verbose_name="record type",
        db_column="REC_TYPE",
        max_length=4,
        db_index=True,
        help_text="Record Type Value: LCCM",
        choices=REC_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=64),
            DocumentCloud(id="2712034", start_page=78),
        ],
    )
    # recip_adr1 = fields.CharField(
    #   max_length=55,
    #   db_column='RECIP_ADR1',
    #   blank=True,
    #   help_text='First line of recipient street address'
    # )
    # recip_adr2 = fields.CharField(
    #   max_length=55,
    #   db_column='RECIP_ADR2',
    #   blank=True,
    #   help_text='Second line of recipient street address'
    # )
    recip_city = fields.CharField(
        max_length=30,
        db_column="RECIP_CITY",
        blank=True,
        help_text="Recipient city",
    )
    recip_id = fields.CharField(
        max_length=9,
        db_column="RECIP_ID",
        blank=True,
        help_text="Recipient identification number",
    )
    recip_namf = fields.CharField(
        max_length=45,
        db_column="RECIP_NAMF",
        blank=True,
        help_text="Recipient first name",
    )
    recip_naml = fields.CharField(
        max_length=200,
        db_column="RECIP_NAML",
        blank=True,
        help_text="Recipient last name",
    )
    recip_nams = fields.CharField(
        max_length=10,
        db_column="RECIP_NAMS",
        blank=True,
        help_text="Recipient name suffix",
    )
    recip_namt = fields.CharField(
        max_length=10,
        db_column="RECIP_NAMT",
        blank=True,
        help_text="Recipient name prefix or title",
    )
    recip_st = fields.CharField(
        max_length=2,
        db_column="RECIP_ST",
        blank=True,
        help_text="Recipient state",
    )
    recip_zip4 = fields.CharField(
        max_length=10,
        db_column="RECIP_ZIP4",
        blank=True,
        help_text="Recipient ZIP Code",
    )
    tran_id = fields.CharField(
        verbose_name="transaction ID",
        max_length=20,
        db_column="TRAN_ID",
        blank=True,
        help_text="Permanent value unique to this item",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "LCCM_CD"
        ordering = ("-ctrib_date",)

    def __str__(self):
        return str(self.filing_id)


class LpayCd(CalAccessBaseModel):
    """
    Payments made or received by lobbying firms.
    """

    UNIQUE_KEY = ("FILING_ID", "AMEND_ID", "LINE_ITEM", "REC_TYPE", "FORM_TYPE")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=107, end_page=109),
        DocumentCloud(id="2711616", start_page=69, end_page=70),
        DocumentCloud(id="2712033", start_page=62),
        DocumentCloud(id="2712034", start_page=76, end_page=77),
    ]
    FILING_FORMS = [
        annotations.get_form("F625").get_section("P2"),
        annotations.get_form("F635").get_section("P3B"),
    ]
    advan_amt = fields.DecimalField(
        decimal_places=2,
        null=True,
        max_digits=14,
        db_column="ADVAN_AMT",
        blank=True,
        help_text="Advance and other payments amount",
    )
    advan_dscr = fields.CharField(
        max_length=100,
        db_column="ADVAN_DSCR",
        blank=True,
        help_text="Description of advance and other payments",
    )
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment identification number. A number of 0 is the \
original filing and 1 to 999 amendments.",
        verbose_name="amendment ID",
    )
    bakref_tid = fields.CharField(
        max_length=20,
        db_column="BAKREF_TID",
        blank=True,
        help_text="Backreference to transaction identifer of parent record",
    )
    cum_total = fields.DecimalField(
        decimal_places=2,
        max_digits=14,
        db_column="CUM_TOTAL",
        help_text="Cumulative total to date",
    )
    # emplr_adr1 = fields.CharField(
    #   max_length=55,
    #   db_column='EMPLR_ADR1',
    #   blank=True,
    #   help_text='First line of employer street address',
    # )
    # emplr_adr2 = fields.CharField(
    #   max_length=55,
    #   db_column='EMPLR_ADR2',
    #   blank=True,
    #   help_text='Second line of employer street address',
    # )
    emplr_city = fields.CharField(
        max_length=30,
        db_column="EMPLR_CITY",
        blank=True,
        help_text="Employer city",
    )
    emplr_id = fields.CharField(
        max_length=9,
        db_column="EMPLR_ID",
        blank=True,
        help_text="This field is undocumented",
    )
    emplr_namf = fields.CharField(
        max_length=45,
        db_column="EMPLR_NAMF",
        blank=True,
        help_text="Employer first name",
    )
    emplr_naml = fields.CharField(
        max_length=200,
        db_column="EMPLR_NAML",
        help_text="Name of firm, employer or coalition",
    )
    emplr_nams = fields.CharField(
        max_length=10,
        db_column="EMPLR_NAMS",
        blank=True,
        help_text="Employer suffix",
    )
    emplr_namt = fields.CharField(
        max_length=10,
        db_column="EMPLR_NAMT",
        blank=True,
        help_text="Employer title or prefix",
    )
    emplr_phon = fields.CharField(
        max_length=20,
        db_column="EMPLR_PHON",
        blank=True,
        help_text="Employer phone number",
    )
    emplr_st = fields.CharField(
        max_length=2,
        db_column="EMPLR_ST",
        blank=True,
        help_text="Employer state",
    )
    emplr_zip4 = fields.CharField(
        max_length=10,
        db_column="EMPLR_ZIP4",
        blank=True,
        help_text="Employer ZIP Code",
    )
    ENTITY_CODE_CHOICES = (
        ("FRM", annotations.choices.LOBBYING_ENTITY_CODES["FRM"]),
        ("LCO", annotations.choices.LOBBYING_ENTITY_CODES["LCO"]),
        ("LEM", annotations.choices.LOBBYING_ENTITY_CODES["LEM"]),
        ("OTH", annotations.choices.LOBBYING_ENTITY_CODES["OTH"]),
        ("128", "Unknown"),
    )
    entity_cd = fields.CharField(
        max_length=3,
        db_column="ENTITY_CD",
        blank=True,
        verbose_name="entity code",
        choices=ENTITY_CODE_CHOICES,
        help_text="Entity Code of the Employer Values",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=62),
            DocumentCloud(id="2712034", start_page=76),
        ],
    )
    fees_amt = fields.DecimalField(
        decimal_places=2,
        null=True,
        max_digits=14,
        db_column="FEES_AMT",
        blank=True,
        help_text="Fees and retainers amount",
    )
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        verbose_name="filing ID",
        help_text="Unique filing identification number",
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        max_length=7,
        db_column="FORM_TYPE",
        verbose_name="form type",
        help_text="Name of the source filing form or schedule",
        db_index=True,
        choices=FORM_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=62),
            DocumentCloud(id="2712034", start_page=76),
        ],
    )
    lby_actvty = fields.CharField(
        max_length=200,
        db_column="LBY_ACTVTY",
        blank=True,
        help_text="Description of lobbying activity",
    )
    line_item = fields.IntegerField(
        db_column="LINE_ITEM",
        help_text="Line item number of this record",
        db_index=True,
    )
    memo_code = fields.CharField(
        max_length=1,
        db_column="MEMO_CODE",
        blank=True,
        help_text="Memo amount flag",
    )
    memo_refno = fields.CharField(
        max_length=20,
        db_column="MEMO_REFNO",
        blank=True,
        help_text="Reference to the text contained in a TEXT record",
    )
    per_total = fields.DecimalField(
        decimal_places=2,
        max_digits=14,
        db_column="PER_TOTAL",
        help_text="Total this reporting period",
    )
    REC_TYPE_CHOICES = (("LPAY", "LPAY"),)
    rec_type = fields.CharField(
        verbose_name="record type",
        db_column="REC_TYPE",
        help_text="Record Type Value: LPAY",
        max_length=4,
        db_index=True,
        choices=REC_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=62),
            DocumentCloud(id="2712034", start_page=76),
        ],
    )
    reimb_amt = fields.DecimalField(
        decimal_places=2,
        null=True,
        max_digits=14,
        db_column="REIMB_AMT",
        blank=True,
        help_text="Reimbursements of expense amount",
    )
    tran_id = fields.CharField(
        verbose_name="transaction ID",
        max_length=20,
        db_column="TRAN_ID",
        blank=True,
        help_text="Permanent value unique to this item",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "LPAY_CD"

    def __str__(self):
        return str(self.filing_id)


class LothCd(CalAccessBaseModel):
    """
    Payments to other lobbying firms.
    """

    UNIQUE_KEY = ("FILING_ID", "AMEND_ID", "LINE_ITEM", "REC_TYPE", "FORM_TYPE")
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614", start_page=106, end_page=107),
        DocumentCloud(id="2711616", start_page=67, end_page=68),
        DocumentCloud(id="2712033", start_page=63),
        DocumentCloud(id="2712034", start_page=77, end_page=78),
    ]
    FILING_FORMS = [
        annotations.get_form("F625").get_section("P3B"),
    ]
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment identification number. A number of 0 is the \
original filing and 1 to 999 amendments.",
        verbose_name="amendment ID",
    )
    amount = fields.DecimalField(
        decimal_places=2,
        null=True,
        max_digits=14,
        db_column="AMOUNT",
        blank=True,
        help_text="Amount of payment",
    )
    cum_amt = fields.DecimalField(
        decimal_places=2,
        null=True,
        max_digits=14,
        db_column="CUM_AMT",
        blank=True,
        help_text="Cumulative total to date",
    )
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        verbose_name="filing ID",
        help_text="Unique filing identification number",
    )
    # firm_adr1 = fields.CharField(
    #   max_length=55,
    #   db_column='FIRM_ADR1',
    #   blank=True,
    #   help_text='First line of firm, employer or coalition\'s address',
    # )
    # firm_adr2 = fields.CharField(
    #   max_length=55,
    #   db_column='FIRM_ADR2',
    #   blank=True,
    #   help_text='Second line of firm, employer or coalition\'s address',
    # )
    firm_city = fields.CharField(
        max_length=30,
        db_column="FIRM_CITY",
        blank=True,
        help_text="Firm, employer or coalition's city",
    )
    firm_name = fields.CharField(
        max_length=200,
        db_column="FIRM_NAME",
        blank=True,
        help_text="Firm, employer or coalition's name",
    )
    firm_phon = fields.CharField(
        max_length=20,
        db_column="FIRM_PHON",
        blank=True,
        help_text="Firm, employer or coalition's phone number",
    )
    firm_st = fields.CharField(
        max_length=2,
        db_column="FIRM_ST",
        blank=True,
        help_text="Firm, employer or coalition's ZIP Code",
    )
    firm_zip4 = fields.CharField(
        max_length=10,
        db_column="FIRM_ZIP4",
        blank=True,
        help_text="Firm ZIP Code",
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        max_length=7,
        db_column="FORM_TYPE",
        verbose_name="form type",
        help_text="Name of the source filing form or schedule",
        db_index=True,
        choices=FORM_TYPE_CHOICES,
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=63),
            DocumentCloud(id="2712034", start_page=77),
        ],
    )
    line_item = fields.IntegerField(
        db_column="LINE_ITEM",
        help_text="Line item number of this record",
        db_index=True,
    )
    memo_code = fields.CharField(
        max_length=1,
        db_column="MEMO_CODE",
        blank=True,
        help_text="Memo amount flag",
    )
    memo_refno = fields.CharField(
        max_length=20,
        db_column="MEMO_REFNO",
        blank=True,
        help_text="Reference to text contained in a TEXT record",
    )
    pmt_date = fields.DateField(
        null=True,
        db_column="PMT_DATE",
        blank=True,
        help_text="Date of payment",
    )
    REC_TYPE_CHOICES = (("LOTH", "LOTH"),)
    rec_type = fields.CharField(
        verbose_name="record type",
        db_column="REC_TYPE",
        max_length=4,
        db_index=True,
        choices=REC_TYPE_CHOICES,
        help_text="Record Type Value: LOTH",
        documentcloud_pages=[
            DocumentCloud(id="2712033", start_page=63),
            DocumentCloud(id="2712034", start_page=77),
        ],
    )
    subj_namf = fields.CharField(
        max_length=45,
        db_column="SUBJ_NAMF",
        blank=True,
        help_text="First name of employer/client subject of lobbying",
    )
    subj_naml = fields.CharField(
        max_length=200,
        db_column="SUBJ_NAML",
        blank=True,
        help_text="Last name of employer/client subject of lobbying",
    )
    subj_nams = fields.CharField(
        max_length=45,
        db_column="SUBJ_NAMS",
        blank=True,
        help_text="Suffix of employer/client subject of lobbying",
    )
    subj_namt = fields.CharField(
        max_length=45,
        db_column="SUBJ_NAMT",
        blank=True,
        help_text="Prefix or title of employer/client subject of lobbying",
    )
    tran_id = fields.CharField(
        verbose_name="transaction ID",
        max_length=20,
        db_column="TRAN_ID",
        blank=True,
        help_text="Permanent value unique to this item",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """

        db_table = "LOTH_CD"
        ordering = ("-pmt_date",)

    def __str__(self):
        return str(self.filing_id)



# Generated by Django 4.0.8 on 2022-10-31 14:44

import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0026_auto_20220909_2321'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cvre530cd',
            name='other_desc',
            field=calaccess_raw.fields.CharField(db_column='OTHER_DESC', help_text='This field is undocumented', max_length=500),
        ),
    ]



import os
import time
import calculate
from github import Github
from django.conf import settings
from calaccess_raw import get_model_list
from calaccess_raw.management.commands import CalAccessCommand
from django.contrib.humanize.templatetags.humanize import intcomma


class Command(CalAccessCommand):
    help = 'Analyze how many model lack a UNIQUE_KEY definition'

    def handle(self, *args, **kwargs):
        """
        Make it happen.
        """
        # Loop through all the models and find any fields without docs
        missing_list = []
        model_count = 0
        for m in get_model_list():
            model_count += 1
            if m.UNIQUE_KEY is None:
                self.log("Missing: %s.%s" % (
                        m().klass_group,
                        m().klass_name,
                    )
                )
                missing_list.append(m)

        # If everything is done, declare victory
        missing_count = len(missing_list)
        if not missing_count:
            self.success("All %s models have a UNIQUE_KEY!" % missing_count)
            return False

        # If not, loop through the missing and create issues
        self.failure(
            "%s/%s (%d%%) of models lack a UNIQUE_KEY" % (
                intcomma(missing_count),
                model_count,
                calculate.percentage(missing_count, model_count)
            )
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-02 21:01
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0001_squashed_0014_auto_20160801_2039"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="rawdataversion",
            options={
                "get_latest_by": "release_datetime",
                "ordering": ("-release_datetime",),
                "verbose_name": "CAL-ACCESS raw data version",
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-26 20:39
from __future__ import unicode_literals

import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0005_auto_20160826_1449"),
    ]

    operations = [
        migrations.AlterField(
            model_name="rcptcd",
            name="amount",
            field=calaccess_raw.fields.DecimalField(
                db_column="AMOUNT",
                decimal_places=2,
                help_text="Amount Received (Monetary, In-kind, Promise)",
                max_digits=14,
            ),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Re-process an archived version of the CAL-ACCESS data.
"""
import os
import logging
from datetime import datetime
import shutil
import zipfile
from hurry.filesize import size
from clint.textui import progress
from django.utils.timezone import now
from django.core.management import call_command
from django.core.management.base import CommandError
from calaccess_raw.management.commands import CalAccessCommand
from calaccess_raw.models.tracking import RawDataVersion
from calaccess_raw import get_data_directory, get_model_list
logger = logging.getLogger(__name__)


class Command(CalAccessCommand):
    """
    Re-process an archived version of the CAL-ACCESS data.
    """
    help = "Re-process an archived version of the CAL-ACCESS data."

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            'release-date',
            help="Date that the version was released (format: YYYY-MM-DD)"
        )
        parser.add_argument(
            "--skip-clean",
            action="store_false",
            dest="clean",
            default=True,
            help="Skip cleaning up the raw data files"
        )
        parser.add_argument(
            "--skip-load",
            action="store_false",
            dest="load",
            default=True,
            help="Skip loading up the raw data files"
        )
        parser.add_argument(
            "--keep-files",
            action="store_true",
            dest="keep_files",
            default=False,
            help="Keep zip, unzipped, TSV and CSV files"
        )
        parser.add_argument(
            "-a",
            "--app-name",
            dest="app_name",
            default="calaccess_raw",
            help="Name of Django app with models into which data will "
                 "be imported (if other not calaccess_raw)"
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # set / compute any attributes that multiple class methods need
        self.release_date = datetime.strptime(
            options['release-date'],
            '%Y-%m-%d'
        ).date()
        self.app_name = options["app_name"]
        self.keep_files = options["keep_files"]
        self.cleaning = options['clean']
        self.loading = options['load']
        self.data_dir = get_download_directory()

        os.path.exists(self.data_dir) or os.makedirs(self.data_dir)

        self.zip_path = os.path.join(self.data_dir, 'calaccess.zip')
        self.tsv_dir = os.path.join(self.data_dir, "tsv/")
        self.csv_dir = os.path.join(self.data_dir, "csv/")

        os.path.exists(self.csv_dir) or os.makedirs(self.csv_dir)
        
        try:
            version = self.raw_data_versions.get(
                release_datetime__year=self.release_date.year,
                release_datetime__month=self.release_date.month,
                release_datetime__day=self.release_date.day,
            )
        except RawDataVersion.DoesNotExist:
            raise CommandError(
                "No release from {0} in archive.".format(self.release_date)
            )

        last_started_reprocess = self.get_last_log()

        self.resume_mode = False

        if last_started_reprocess:
            if last_started_reprocess.finish_datetime:
                if last_started_reprocess.version == version:
                    if len(last_started_reprocess.called) > 0:
                        self.log('Resuming previous re-processing job.')
                        self.resume_mode = True

        if self.resume_mode:
            self.log_record = last_started_reprocess
        else:
            # create a new log record
            self.log_record = self.command_logs.create(
                version=version,
                command=self,
                called_by=self.get_caller_log()
            )

        if self.verbosity:
            self.header("Copying archived zip file to data directory")

        if not self.resume_mode:
            # copy the zip file to the data directory
            with open(self.zip_path, 'w') as local_zip:
                version.zip_file_archive.open()
                local_zip.write(version.zip_file_archive.read())
                version.zip_file_archive.close()

            self.unzip()
            self.prep()

            if self.verbosity:
                self.duration()

        # execute the other steps that haven't been skipped
        if options['clean']:
            self.clean()
            if self.verbosity:
                self.duration()

        if options['load']:
            self.load()
            if self.verbosity:
                self.duration()

        self.log_record.finish_datetime = now()
        self.log_record.save()

        if self.verbosity:
            self.success("Done!")
        logger.info("Done!")

    def unzip(self):
        """
        Unzip the snapshot file.
        """
        if self.verbosity:
            self.log(" Unzipping archive")

        with zipfile.ZipFile(self.zip_path) as zf:
            for member in zf.infolist():
                words = member.filename.split('/')
                path = self.data_dir
                for word in words[:-1]:
                    drive, word = os.path.splitdrive(word)
                    head, word = os.path.split(word)
                    if word in (os.curdir, os.pardir, ''):
                        continue
                    path = os.path.join(path, word)
                zf.extract(member, path)

    def prep(self):
        """
        Rearrange the unzipped files and get rid of the stuff we don't want.
        """
        if self.verbosity:
            self.log(" Prepping unzipped data")

        # Move the deep down directory we want out
        shutil.move(
            os.path.join(
                self.data_dir,
                'CalAccess/DATA/CalAccess/DATA/'
            ),
            self.data_dir
        )
        # Clear out target if it exists
        if os.path.exists(self.tsv_dir):
            shutil.rmtree(self.tsv_dir)

        # Rename it to the target
        shutil.move(
            os.path.join(self.data_dir, "DATA/"),
            self.tsv_dir,
        )

    def clean(self):
        """
        Clean up the raw data files from the state so they are ready to get loaded into the database.
        """
        if self.verbosity:
            self.header("Cleaning data files")

        tsv_list = os.listdir(self.tsv_dir)

        if self.resume_mode:
            # get finished clean command logs of last update
            prev_cleaned = [
                x.file_name + '.TSV'
                for x in self.log_record.called.filter(
                    command='cleancalaccessrawfile',
                    finish_datetime__isnull=False
                )
            ]
            self.log("{} files already cleaned.".format(len(prev_cleaned)))
            # remove these from tsv_list
            tsv_list = [x for x in tsv_list if x not in prev_cleaned]

        # Loop through all the files in the source directory
        if self.verbosity:
            tsv_list = progress.bar(tsv_list)
        for name in tsv_list:
            call_command(
                "cleancalaccessrawfile",
                name,
                verbosity=self.verbosity,
                keep_files=self.keep_files,
            )

    def load(self):
        """
        Loads the cleaned up csv files into the database.
        """
        if self.verbosity:
            self.header("Loading data files")

        model_list = [
            x for x in get_model_list() if os.path.exists(x.objects.get_csv_path())
        ]

        if self.resume_mode:
            # get finished load command logs of last update
            prev_loaded = [
                x.file_name
                for x in self.log_record.called.filter(
                    command='loadcalaccessrawfile',
                    finish_datetime__isnull=False
                )
            ]
            self.log("{} models already loaded.".format(len(prev_loaded)))
            # remove these from model_list
            model_list = [x for x in model_list if x._meta.db_table not in prev_loaded]

        if self.verbosity:
            model_list = progress.bar(model_list)
        for model in model_list:
            call_command(
                "loadcalaccessrawfile",
                model.__name__,
                verbosity=self.verbosity,
                keep_files=self.keep_files,
                app_name=self.app_name,
            )



import os
import time
import calculate
from github import Github
from django.conf import settings
from calaccess_raw import get_model_list
from calaccess_raw.management.commands import CalAccessCommand
from django.contrib.humanize.templatetags.humanize import intcomma


class Command(CalAccessCommand):
    help = 'Create GitHub issues for model fields without documentation'

    def set_options(self, *args, **kwargs):
        """
        Hook up with the GitHub API and prepare to create issues.
        """
        self.gh = Github(os.getenv('GITHUB_TOKEN'))
        self.org = self.gh.get_organization("california-civic-data-coalition")
        self.repo = self.org.get_repo("django-calaccess-raw-data")
        self.labels = [
            self.repo.get_label("small"),
            self.repo.get_label("documentation"),
            self.repo.get_label("enhancement"),
        ]
        self.milestone = self.repo.get_milestone(3)

    def handle(self, *args, **kwargs):
        """
        Make it happen.
        """
        self.set_options()
        self.header(
            "Creating GitHub issues for model fields without documentation"
        )

        # Loop through all the models and find any fields without docs
        field_count = 0
        missing_list = []
        for m in get_model_list():
            field_list = m().get_field_list()
            field_count += len(field_list)
            for f in field_list:
                if not self.has_docs(f):
                    missing_list.append((m, f))

        # If everything is done, declare victory
        if not missing_list:
            self.success("All %s fields documented!" % field_count)
            return False

        # If not, loop through the missing and create issues
        missing_count = len(missing_list)
        self.log(
            "- %s/%s (%d%%) of fields lack documentation" % (
                intcomma(missing_count),
                intcomma(field_count),
                calculate.percentage(missing_count, field_count)
            )
        )
        for model, field in missing_list[611:]:
            # For now we are excluding the 'other' model module to
            # avoid overkill
            if model().klass_group != 'other':
                self.create_issue(model, field)

    def has_docs(self, field):
        """
        Test if a Django field has some kind of documentation already.

        Returns True or False
        """
        if field.name == 'id':
            return True
        if field.help_text:
            return True
        if field.__dict__['_verbose_name']:
            return True
        return False

    def create_issue(self, model, field):
        """
        Create a GitHub issue for the provided model and field.
        """
        title = TITLE_TEMPLATE % (field.name, model().klass_name)

        body = BODY_TEMPLATE % (
               field.name,
               model().klass_name,
               model().klass_group,
               model().klass_group,
        )
        self.log("-- Creating issue for %s.%s" % (
                model().klass_name,
                field.name
            )
        )
        self.repo.create_issue(
            title,
            body=body,
            labels=self.labels,
            milestone=self.milestone
        )
        time.sleep(2.5)

TITLE_TEMPLATE = """
Add documentation for the ``%s`` field on the ``%s`` database model
""".replace("\n", "")

BODY_TEMPLATE = """
## Your mission

Add documentation for the ``%s`` field on the ``%s`` database model.

## Here's how

**Step 1**: Claim this ticket by leaving a comment below. Tell everyone you're ON IT!

**Step 2**: Open up the file that contains this model. It should be in <a href="https://github.com/california-civic-data-coalition/django-calaccess-raw-data/blob/master/calaccess_raw/models/%s.py">calaccess_raw.models.%s.py</a>.

**Step 3**: Hit the little pencil button in the upper-right corner of the code box to begin editing the file.

![Edit](https://dl.dropboxusercontent.com/u/3640647/ScreenCloud/1440367320.67.png)

**Step 4**: Find this model and field in the file. (Clicking into the box and searching with CTRL-F can help you here.) Once you find it, we expect the field to lack the ``help_text`` field typically used in Django to explain what a field contains.

```python
effect_dt = fields.DateField(
    null=True,
    db_column="EFFECT_DT"
)
```

**Step 5**: In a separate tab, open up the <a href="Quilmes">official state documentation</a> and find the page that defines all the fields in this model.

![The docs](https://dl.dropboxusercontent.com/u/3640647/ScreenCloud/1440367001.08.png)

**Step 6**: Find the row in that table's definition table that spells out what this field contains. If it lacks documentation. Note that in the ticket and close it now.

![The definition](https://dl.dropboxusercontent.com/u/3640647/ScreenCloud/1440367068.59.png)

**Step 7**: Return to the GitHub tab.

**Step 8**: Add the state's label explaining what's in the field, to our field definition by inserting it a ``help_text`` argument. That should look something like this:

```python
effect_dt = fields.DateField(
    null=True,
    db_column="EFFECT_DT",
    # Add a help_text argument like the one here, but put your string in instead.
    help_text="The other values in record were effective as of this date"
)
```

**Step 9**: Scroll down below the code box and describe the change you've made in the commit message. Press the button below.

![Commit](https://dl.dropboxusercontent.com/u/3640647/ScreenCloud/1440367511.66.png)

**Step 10**:  Review your changes and create a pull request submitting them to the core team for inclusion.

![Pull request](https://dl.dropboxusercontent.com/u/3640647/ScreenCloud/1440368058.52.png)

That's it! Mission accomplished!
"""



# Generated by Django 3.2.4 on 2021-06-14 16:27

from django.db import migrations
import ia_storage.fields
import ia_storage.storage


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0021_auto_20210614_1604"),
    ]

    operations = [
        migrations.AlterField(
            model_name="rawdatafile",
            name="clean_file_archive",
            field=ia_storage.fields.InternetArchiveFileField(
                blank=True,
                help_text="An archive of the raw data file after being cleaned.",
                max_length=255,
                storage=ia_storage.storage.InternetArchiveStorage,
                upload_to="",
                verbose_name="archive of clean file",
            ),
        ),
        migrations.AlterField(
            model_name="rawdatafile",
            name="error_log_archive",
            field=ia_storage.fields.InternetArchiveFileField(
                blank=True,
                help_text="An archive of the error log containing lines from the original download file that could not be parsed and are excluded from the cleaned file.",
                max_length=255,
                storage=ia_storage.storage.InternetArchiveStorage,
                upload_to="",
                verbose_name="archive of error log",
            ),
        ),
    ]



from django.conf import settings
from django.contrib import admin
from django.urls import re_path
from django.views.static import serve
admin.autodiscover()


urlpatterns = (
    re_path(r'^admin/', admin.site.urls),
    re_path(r'^static/(?P<path>.*)$', serve, {
        'document_root': settings.STATIC_ROOT,
        'show_indexes': True,
    }),
)



# Generated by Django 4.1.13 on 2024-07-12 12:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0027_alter_cvre530cd_other_desc"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="cvrf470cd",
            name="cand_adr1",
        ),
    ]



import os
import shutil
from datetime import datetime
from itertools import chain
from optparse import make_option
from clint.textui import progress
from subsample.file_input import FileInput
from subsample.algorithms import two_pass_sample
from django.conf import settings
from calaccess_raw.management.commands import CalAccessCommand
from calaccess_raw.models import RawDataVersion
from zipfile import ZIP_DEFLATED, ZIP_STORED, ZipFile


class Command(CalAccessCommand):
    help = 'Create smaller sampled TSV files for unit tests'

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "--sample-rows",
            action="store",
            dest="samplerows",
            default=1000,
            help="Number of rows to grab from each table"
        )
    
    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)

        # Set options
        self.test_data_dir = os.path.join(settings.BASE_DIR, 'test-data')
        self.sample_dir = os.path.join(self.test_data_dir, "tsv/")
        self.sample_rows = int(options['samplerows'])
        self.tsv_list = os.listdir(self.tsv_dir)

        self.header("Sampling %i rows from %s source files" % (
            self.sample_rows,
            len(self.tsv_list),
        ))

        # Make sure sample dir exists and is empty
        os.path.exists(self.test_data_dir) or os.makedirs(self.test_data_dir)
        os.path.exists(self.sample_dir) and shutil.rmtree(self.sample_dir)
        os.makedirs(self.sample_dir)

        # Loop through all the files in the source directory
        for name in progress.bar(self.tsv_list):

            # Find the input
            file = os.path.join(self.tsv_dir, name)
            out_file = os.path.join(self.sample_dir, name)

            if self.verbosity > 2:
                self.log(" Sampling %s" % file)

            # Open the file
            fi = FileInput(file, True)

            # Generate our sample
            sample = two_pass_sample(fi, sample_size=self.sample_rows)

            # Open our output file
            with open(out_file, 'wb') as out:

                # Write it out
                for line in chain(fi.header, sample):
                    out.write(line)

        self.header("Compressing zip file...")
        self.save_zip()
    
    def save_zip(self):
        """
        Save a zip file containing all the sampled .TSV files
        """
        # enable zipfile compression
        compression = ZIP_DEFLATED
        zip_path = os.path.join(self.test_data_dir, 'dbwebexport.zip')

        try:
            zf = ZipFile(zip_path, 'w', compression, allowZip64=True)
        except RuntimeError:
            logger.error('Zip file cannot be compressed (check zlib module).')
            compression = ZIP_STORED
            zf = ZipFile(zip_path, 'w', compression, allowZip64=True)

        # loop over and save files in csv dir
        for name in os.listdir(self.sample_dir):
            if self.verbosity > 2:
                self.log(" Adding %s to zip" % name)
            f = os.path.join(self.sample_dir, name)
            zf.write(f, 'CalAccess/DATA/' + name)

        zf.close()



"""
Tests for the models.py files.
"""
# Testing
from django.test import TestCase

# Stuff to test
from calaccess_raw import models
from calaccess_raw.admin import BaseAdmin
from django.contrib.admin import AdminSite

# Python misc.
from copy import deepcopy

# Logging
import logging

logger = logging.getLogger(__name__)


class ModelTestCase(TestCase):
    """
    Tests related to the database models.
    """

    multi_db = True

    def test_model_methods(self):
        """
        Test the extra methods we've added to models.
        """
        # CAL-ACCESS models
        m = models.RcptCd()
        m.__str__()
        m.doc()
        m.__doc__ = "RcptCd"
        m.doc()
        m.db_table
        m.klass
        m.get_tsv_name()
        m.get_tsv_path()
        m.get_unique_key_list()
        m.get_documentcloud_pages()
        m.get_filing_forms_w_sections()
        unique_key = deepcopy(models.RcptCd.UNIQUE_KEY)
        models.RcptCd.UNIQUE_KEY = None
        models.RcptCd().get_unique_key_list()
        models.RcptCd.UNIQUE_KEY = unique_key

    def test_admins(self):
        """
        Text the extra methods we've added to the admins.
        """
        BaseAdmin(models.RcptCd, AdminSite()).get_readonly_fields()



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-10-28 01:29
from __future__ import unicode_literals

import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0010_auto_20161026_1643"),
    ]

    operations = [
        migrations.AlterField(
            model_name="expncd",
            name="bakref_tid",
            field=calaccess_raw.fields.CharField(
                blank=True,
                db_column="BAKREF_TID",
                help_text="Back Reference to a Tran_ID of a 'parent' record",
                max_length=20,
                verbose_name="back reference transaction id",
            ),
        ),
        migrations.AlterField(
            model_name="expncd",
            name="bal_juris",
            field=calaccess_raw.fields.CharField(
                blank=True,
                db_column="BAL_JURIS",
                help_text="Ballot measure's jurisdiction",
                max_length=40,
                verbose_name="ballot measure jurisdiction",
            ),
        ),
        migrations.AlterField(
            model_name="expncd",
            name="bal_name",
            field=calaccess_raw.fields.CharField(
                blank=True,
                db_column="BAL_NAME",
                help_text="Ballot Measure Name",
                max_length=200,
                verbose_name="ballot measure name",
            ),
        ),
        migrations.AlterField(
            model_name="expncd",
            name="bal_num",
            field=calaccess_raw.fields.CharField(
                blank=True,
                db_column="BAL_NUM",
                help_text="Ballot Number or Letter",
                max_length=7,
                verbose_name="ballot measure number",
            ),
        ),
        migrations.AlterField(
            model_name="expncd",
            name="expn_dscr",
            field=calaccess_raw.fields.CharField(
                blank=True,
                db_column="EXPN_DSCR",
                help_text="Purpose of expense and/or description/explanation",
                max_length=400,
                verbose_name="expense description",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-21 18:21
from __future__ import unicode_literals

import calaccess_raw.annotations
import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0013_auto_20161123_2219"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="rawdatafile",
            options={
                "ordering": ("-version_id", "file_name"),
                "verbose_name": "TRACKING: CAL-ACCESS raw data file",
            },
        ),
        migrations.AlterModelOptions(
            name="rawdataversion",
            options={
                "get_latest_by": "release_datetime",
                "ordering": ("-release_datetime",),
                "verbose_name": "TRACKING: CAL-ACCESS raw data version",
            },
        ),
        migrations.AlterField(
            model_name="rcptcd",
            name="tran_type",
            field=calaccess_raw.fields.CharField(
                blank=True,
                choices=[
                    ("F", "Forgiven Loan"),
                    ("I", "Intermediary"),
                    ("R", "Returned"),
                    ("T", "Third Party Repayment"),
                    ("X", "Transfer"),
                    ("0", "Unknown"),
                    ("M", "Unknown"),
                    ("N", "Unknown"),
                ],
                db_column="TRAN_TYPE",
                documentcloud_pages=[
                    calaccess_raw.annotations.DocumentCloud(
                        id="2711616-MapCalFormat2Fields", start_page=72
                    ),
                    calaccess_raw.annotations.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=29
                    ),
                    calaccess_raw.annotations.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=38
                    ),
                ],
                help_text="Transaction Type",
                max_length=1,
                verbose_name="transaction type",
            ),
        ),
    ]



# Generated by Django 3.2.4 on 2021-06-13 22:58

from django.db import migrations
import ia_storage.fields
import ia_storage.storage


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0018_auto_20210426_2015"),
    ]

    operations = [
        migrations.AlterField(
            model_name="rawdataversion",
            name="download_zip_archive",
            field=ia_storage.fields.InternetArchiveFileField(
                blank=True,
                help_text="An archive of the original zipped file downloaded from CAL-ACCESS.",
                max_length=255,
                storage=ia_storage.storage.InternetArchiveStorage,
                upload_to="",
                verbose_name="download files zip file",
            ),
        ),
    ]



# Generated by Django 2.1.15 on 2020-08-18 05:42

import calaccess_raw.annotations.documents
import calaccess_raw.annotations.forms
import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0015_auto_20170729_0218"),
    ]

    operations = [
        migrations.AlterField(
            model_name="cvr3verificationinfocd",
            name="sig_naml",
            field=calaccess_raw.fields.CharField(
                blank=True,
                db_column="SIG_NAML",
                help_text="last name of the signer",
                max_length=500,
                verbose_name="last name",
            ),
        ),
    ]



"""
Base management command that provides common functionality for the other commands in this app.
"""
import logging
import os
from re import sub
from datetime import datetime
from django.utils.termcolors import colorize
from django.core.management.base import BaseCommand
from calaccess_raw import get_data_directory

logger = logging.getLogger(__name__)


class CalAccessCommand(BaseCommand):
    """
    Base management command that provides common functionality for the other commands in this app.
    """

    url = "https://campaignfinance.cdn.sos.ca.gov/dbwebexport.zip"

    def handle(self, *args, **options):
        """
        The generic handler function.

        Any command subclassing this object should implement its own
        handle method, as is standard in Django, and run this method
        via a super call to inherit its functionality.
        """
        self.set_global_options(options)

    def set_global_options(self, options):
        """
        Set options to all commands.
        """
        # Set global options
        self.verbosity = options.get("verbosity")
        self.no_color = options.get("no_color")

        # set up data directories
        self.data_dir = get_data_directory()
        self.tsv_dir = os.path.join(self.data_dir, "tsv")
        self.csv_dir = os.path.join(self.data_dir, "csv")

        os.path.exists(self.data_dir) or os.makedirs(self.data_dir)
        os.path.exists(self.tsv_dir) or os.makedirs(self.tsv_dir)
        os.path.exists(self.csv_dir) or os.makedirs(self.csv_dir)

        # set path where zip will be downloaded
        self.download_dir = os.path.join(self.data_dir, "download")
        self.zip_path = os.path.join(self.download_dir, self.url.split("/")[-1])

        # Start the clock
        self.start_datetime = datetime.now()

    #
    # Logging methods
    #

    def header(self, string):
        """
        Writes out a string to stdout formatted to look like a header.
        """
        logger.debug(string)
        if not self.no_color:
            string = colorize(string, fg="cyan", opts=("bold",))
        self.stdout.write(string)

    def log(self, string):
        """
        Writes out a string to stdout formatted to look like a standard line.
        """
        logger.debug(string)
        if not self.no_color:
            string = colorize("%s" % string, fg="white")
        self.stdout.write(string)

    def success(self, string):
        """
        Writes out a string to stdout formatted green to communicate success.
        """
        logger.debug(string)
        if not self.no_color:
            string = colorize(string, fg="green")
        self.stdout.write(string)

    def failure(self, string):
        """
        Writes out a string to stdout formatted red to communicate failure.
        """
        logger.debug(string)
        if not self.no_color:
            string = colorize(string, fg="red")
        self.stdout.write(string)

    def duration(self):
        """
        Calculates how long the command has been running and writes it to stdout.
        """
        duration = datetime.now() - self.start_datetime
        self.stdout.write("Duration: {}".format(str(duration)))
        logger.debug("Duration: {}".format(str(duration)))

    def __str__(self):
        return sub(r"(.+\.)*", "", self.__class__.__module__)
